
C.........................................................................
C Version "@(#)$Header$"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-2002 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        LOGICAL   FUNCTION  CHECK3 ( FNAME, VNAME, JDATE, JTIME )

C***********************************************************************
C  function body starts at line  93
C
C  FUNCTION:  returns TRUE iff time step for (JDATE,JTIME) and variable
C       VNAME is available in file with logical name FNAME.  If FNAME 
C       is time idndependent or is a dictionary file, only VNAME is 
C       significant and the routine returns the availability of the
C       variable or file description for VNAME.
C
C  PRECONDITIONS REQUIRED:  
C       FNAME already opened by OPEN3().
C       VNAME a valid variable in FNAME, or else is ALLVAR3=='ALL'
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C
C  REVISION  HISTORY:  
C       prototype 3/1992 by CJC
C
C       Modified  7/1994 by CJC to handle restart (circular-buffer) files
C
C       modified 10/1994 by CJC to work with variable-level-granularity
C       WRITE3() 
C
C       modified 19/1999 by C.Coats for thread-safety, A.Trayanov for
C       coupling-mode operation
C
C       Modified  1/2002 by CJC:  check TRIMLEN() of FNAME
C
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME   !  file name for query
        CHARACTER*(*)   VNAME   !  vble name for query
        INTEGER         JDATE   !  date (YYYYDDD) for query
        INTEGER         JTIME   !  time (HHMMSS) for query


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         INDEX1     !  look up names in name tables
        INTEGER         INIT3      !  initialize I/O system files.
        INTEGER         JSTEP3     !  compute time step record numbers
        INTEGER         TRIMLEN    !  trimmed string length

        EXTERNAL        INDEX1, INIT3, JSTEP3, TRIMLEN
        EXTERNAL        INITBLK3   !  block data: initialize I/O state


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         FID             !  subscript for STATE3 arrays
        INTEGER         STEP            !  time step record number
        INTEGER         STEP2           !  1 or 2, according to step mod 2
        INTEGER         IERR            !  netCDF error status return
        INTEGER         DIMT( 5 )       !  corner   for NCVGT()
        INTEGER         DELT( 5 )       !  diagonal for NCVGT()
        INTEGER         VID             !  subscript for vble in state arrays
        INTEGER         V               !  "; loop counter over variables
        INTEGER         F1, F2          !  test-values for FLAGS
        INTEGER         FLAGS( 2,MXVARS3 )!  flags from NCVGT()            
        CHARACTER*16    FIL16           !  scratch file-name-buffer
        CHARACTER*16    VAR16           !  scratch vble-name-buffer
        LOGICAL         EFLAG
        CHARACTER*256   MESG

#ifdef IOAPICPL
        INTEGER       COUNT, SKIP, TYPE
        REAL          BUFFER(1)
        LOGICAL       READ3V
        EXTERNAL      READ3V
#endif

C***********************************************************************
C   begin body of function  CHECK3

        EFLAG = .FALSE.
!$OMP   CRITICAL( S_LOGDEV )
        IF ( .NOT. FINIT3 ) THEN
            LOGDEV = INIT3()
            WRITE( LOGDEV, '( 5X, A )' )
     &       'WARNING in CHECK3: I/O API not yet initialized.'
            CHECK3 = .FALSE.
            EFLAG  = .TRUE.
        END IF          !  if not FINIT3
!$OMP   END CRITICAL( S_LOGDEV )
        IF ( EFLAG ) RETURN

        EFLAG = .FALSE.
        IF ( TRIMLEN( FNAME ) .GT. NAMLEN3 ) THEN
            EFLAG = .TRUE.
            MESG  = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A , I10 )' )
     &          'Max file name length 16; actual:', TRIMLEN( FNAME )
            CALL M3MSG2( MESG )
        END IF          !  if len( fname ) > 16

        IF ( TRIMLEN( VNAME ) .GT. NAMLEN3 ) THEN
            EFLAG = .TRUE.
            MESG  = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I10 )'  )
     &          'Max vble name length 16; actual:', TRIMLEN( VNAME )
            CALL M3MSG2( MESG )
        END IF          !  if len( vname ) > 16
        
        IF ( EFLAG ) THEN
            MESG = 'Invalid variable or file name arguments'
            CALL M3WARN( 'CHECK3', JDATE, JTIME, MESG )
	    CHECK3 = .FALSE.
            RETURN
        END IF
        

C.......   Find netCDF index for the file, and check time step availability:

        FIL16 = FNAME   !  fixed-length-16 scratch copy of name
        VAR16 = VNAME   !  "
        FID   = INDEX1( FIL16, COUNT3, FLIST3 )

        IF ( FID .EQ. 0 ) THEN
            
            CHECK3 = .FALSE.
            MESG = 'File "' // FNAME // '" not yet open.'
            CALL M3WARN( 'CHECK3', JDATE, JTIME, MESG )
            RETURN
            
        ELSE IF ( FTYPE3( FID ) .EQ. DCTNRY3 ) THEN
            
            STEP  = INDEX1( VAR16, NVARS3( FID ), VLIST3( 1,FID ) )
            STEP2 = STEP
            VID   = STEP
            
        ELSE
            
            STEP = JSTEP3( JDATE, JTIME,
     &                     SDATE3( FID ),
     &                     STIME3( FID ),
     &                     ABS( TSTEP3( FID ) ) )
            
            IF ( VAR16 .NE. ALLVAR3 ) THEN
                VID = INDEX1( VAR16, NVARS3( FID ), VLIST3( 1,FID ) )
                IF ( VID .EQ. 0 ) THEN
                    CHECK3 = .FALSE.
                    MESG = 'Variable "' // VNAME //
     &                     '" not in file "' // FNAME
                    CALL M3WARN( 'CHECK3', JDATE, JTIME, MESG )
                    RETURN
                END IF
            ELSE
                VID = ALLAYS3
            END IF              !  if "all variables", or not
         
            IF ( STEP .LT. 0 ) THEN
                   
                CHECK3 = .FALSE.
                RETURN
                
            END IF
                   
            IF ( TSTEP3( FID ) .LT. 0 ) THEN
                STEP2 = 1 + MOD( STEP-1, 2 )
                F1    = JDATE
                F2    = JTIME
            ELSE IF ( TSTEP3( FID ) .GT. 0 ) THEN
                F1    = JDATE
                F2    = JTIME
                STEP2 = STEP
            ELSE
                F1    = 0
                F2    = 0
                STEP2 = STEP
            END IF
             
        END IF          !  if file not open, or if dictionary, or ...
        
#ifdef IOAPICPL
        IF ( CDFID3( FID ) .EQ. VIRFIL3 ) THEN     !  virtual "file"
           COUNT = 0
           SKIP  = 0
           IF ( VAR16 .EQ. ALLVAR3 ) THEN
              EFLAG = .TRUE.
              DO VID = 1, NVARS3(FID)
                 TYPE = VTYPE3(VID,FID)
                 EFLAG = EFLAG .AND.
     $                READ3V(PLIST3(FID),VLIST3(VID,FID),SKIP,
     $                       F1,F2,BUFFER,COUNT,TYPE)
              ENDDO
              CHECK3 = EFLAG
           ELSE
              TYPE = VTYPE3(VID,FID)
              CHECK3 = READ3V(PLIST3(FID),VNAME,SKIP,
     $                        F1,F2,BUFFER,COUNT,TYPE)
           ENDIF

           RETURN
        
        END IF  
#endif

        IF ( VOLAT3( FID ) ) THEN      !  volatile file:  synch with disk
           
!$OMP CRITICAL( S_NC )
            CALL NCSNC( CDFID3( FID ), IERR )
!$OMP END CRITICAL( S_NC )
            IF ( IERR .NE. 0 ) THEN

                WRITE( MESG, '( A, I7, 2X, A )' )
     &              'netCDF error number', IERR,
     &              'Error with disk synchronization for file:  '
     &              // FIL16
                CALL M3WARN( 'CHECK3', JDATE, JTIME, MESG )
                CHECK3 = .FALSE.
                RETURN
           
            END IF              !  if synch failed
           
        END IF                  !  if file is volatile
        
        DIMT( 1 ) = 1           !  field:  date and time
        DELT( 1 ) = 2           !  extent:  both date and time
        DIMT( 3 ) = STEP2       !  timestep record number
        DELT( 3 ) = 1           !  extent:  1 time step
        
        IF ( VID .GT. 0 ) THEN      !  checking just one variable
            
            DIMT( 2 ) = VID     !  initial variable number
            DELT( 2 ) = 1       !  extent:  one variable
            
!$OMP       CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ), TINDX3( FID ), 
     &                  DIMT, DELT, FLAGS( 1,VID ), IERR )
!$OMP       END CRITICAL( S_NC )
            
            IF ( IERR .EQ. 0 ) THEN
                CHECK3 = ( FLAGS( 1,VID ) .EQ. F1  .AND.
     &                     FLAGS( 2,VID ) .EQ. F2  )
            ELSE IF ( IERR .EQ. NCECOORD ) THEN
                CHECK3 = .FALSE.
            ELSE                !  ierr nonzero:
                WRITE( MESG, '( 5A, I7 )' )
     &              'Error reading time step flag for file "',
     &              FNAME, '" variable "', VNAME,
     &              '" netCDF error number', IERR
                CALL M3WARN( 'CHECK3', JDATE, JTIME, MESG )
                CHECK3 = .FALSE.
            END IF              !  if ierr = 0  or  not.
        
        ELSE                    !  checking all variables
                
            DIMT( 2 ) = 1               !  initial variable number
            DELT( 2 ) = NVARS3( FID )   !  extent:  all variables
            
!$OMP       CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ), TINDX3( FID ), 
     &                  DIMT, DELT, FLAGS, IERR )
!$OMP       END CRITICAL( S_NC )
            
            IF ( IERR .EQ. NCECOORD ) THEN
                CHECK3 = .FALSE.
                RETURN
            ELSE IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 3A, I7 )' )
     &              'Error reading time step flag for file "', FNAME,
     &              '" netCDF error number', IERR
                CALL M3WARN( 'CHECK3', JDATE, JTIME, MESG )
                CHECK3 = .FALSE.
                RETURN
            END IF              !  if ncvgt() failed
        
            DO  11  V = 1, NVARS3( FID )
            
                IF ( FLAGS( 1,V ) .NE. F1  .OR. 
     &               FLAGS( 2,V ) .NE. F2       ) THEN
                    CHECK3 = .FALSE.
                    RETURN
                END IF  !  if ierr = 0  or  not.
            
11          CONTINUE
                
            CHECK3 = .TRUE.             ! (if you get to here)
        
        END IF      !  if checking just one vble, or if checking all vbles

        RETURN

        END

