
C.........................................................................
C Version "@(#)$Header$"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-2002 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        LOGICAL   FUNCTION  DESC3 ( FNAME )

C***********************************************************************
C  function body starts at line 84
C
C  FUNCTION:  puts description of FNAME into file description commons,
C             found in include file FDESC3.EXT.
C
C  RETURN VALUE:  TRUE iff successful
C
C  PRECONDITIONS REQUIRED:  Models-3 file with logical name FNAME must
C             have already opened by OPEN3() or CREATE3()
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C
C  REVISION  HISTORY:
C	prototype 3/1992 by CJC
C
C	modified  7/1994 by CJC to return new coordinate-system and
C                             grid-description parameters
C
C	Modified 10/1996 by CJC to support file type TSERIES3
C
C       Modified  5/1998 by CJC for OpenMP thread-safety
C
C       Modified  5/1999 by ALT for coupling-mode operation

C       Modified  1/2002 by CJC:  check TRIMLEN() of FNAME
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'FDESC3.EXT'
        INCLUDE 'ATDSC3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME   !  logical name of file to be described.


C...........   EXTERNAL FUNCTIONS & BLOCK DATA

        INTEGER         INDEX1     !  look up names in name tables
        INTEGER         INIT3      !  initialize I/O system files.
        INTEGER         TRIMLEN    !  trimmed string length

        EXTERNAL        INDEX1, INIT3, TRIMLEN
        EXTERNAL        INITBLK3   !  block data: initialize I/O state


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         FID             !  subscript for STATE3 arrays
        INTEGER         FNUM            !  netCDF file ID from NCCRE()
        INTEGER         IERR            !  netCDF error status return
        INTEGER         VAR, LVL, ATT   !  loop counter for file variables
        CHARACTER*16    FIL16           !  scratch name-buffer
        CHARACTER*256   MESG            !  message-buffer
        LOGICAL         EFLAG

#ifdef IOAPICPL
        LOGICAL       DESC3V
        EXTERNAL      DESC3V
#endif


C.............................................................................
C   begin body of subroutine  OPEN3

        EFLAG = .FALSE.
!$OMP   CRITICAL( S_LOGDEV )
        IF ( .NOT. FINIT3 ) THEN
            LOGDEV = INIT3()
            WRITE( LOGDEV, '( 5X, A )' )
     &       'WARNING in DESC3: I/O API not yet initialized.'
            DESC3 = .FALSE.
            EFLAG = .TRUE.
        END IF          !  if not FINIT3
!$OMP   END CRITICAL( S_LOGDEV )
        IF ( EFLAG ) RETURN

        IF ( TRIMLEN( FNAME ) .GT. NAMLEN3 ) THEN
            MESG = 'File "'// FNAME // '"'
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I10 )' )
     &          'Max file name length 16; actual:', TRIMLEN( FNAME )
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF


C.......   Find netCDF index for the file, and check time step availability:

        FIL16 = FNAME   !  fixed-length-16 scratch copy of name
        FID   = INDEX1( FIL16, COUNT3, FLIST3 )

        IF ( FID .EQ. 0 ) THEN
            DESC3 = .FALSE.
            RETURN
        END IF


#ifdef IOAPICPL
        IF ( CDFID3( FID ) .EQ. VIRFIL3 ) THEN     !  VIRTUAL "file"
           DESC3  = DESC3V(PLIST3(FID))
           RETURN
        END IF  
#endif

C.......   Characteristics stored in STATE3.EXT:

        FTYPE3D = FTYPE3( FID )
        FNUM    = CDFID3( FID )

        NVARS3D = NVARS3( FID )
        DO  111  VAR = 1, NVARS3D
            VNAME3D( VAR ) = VLIST3( VAR,FID )
111     CONTINUE

        IF ( FTYPE3D .NE. DCTNRY3 ) THEN

            SDATE3D = SDATE3( FID )
            STIME3D = STIME3( FID )
            TSTEP3D = TSTEP3( FID )
            MXREC3D = MXREC3( FID )
            NTHIK3D = NTHIK3( FID )
            NCOLS3D = NCOLS3( FID )
            NROWS3D = NROWS3( FID )
            NLAYS3D = NLAYS3( FID )
            GDTYP3D = GDTYP3( FID )
            P_ALP3D = P_ALP3( FID )
            P_BET3D = P_BET3( FID )
            P_GAM3D = P_GAM3( FID )
            XCENT3D = XCENT3( FID )
            YCENT3D = YCENT3( FID )
            XORIG3D = XORIG3( FID )
            YORIG3D = YORIG3( FID )
            XCELL3D = XCELL3( FID )
            YCELL3D = YCELL3( FID )
            GDNAM3D = GDNAM3( FID )

            DO  112  VAR = 1, NVARS3D
                VTYPE3D( VAR ) = VTYPE3( VAR,FID )
112         CONTINUE

        END IF  !  if not a dictionary file


C.......   If buffered file, do not know other attributes:

        IF ( FNUM .EQ. BUFFIL3 ) THEN

            EXECN3D = CMISS3
            VGTYP3D = IMISS3
            VGTOP3D = BADVAL3
            CDATE3D = CURDATE
            CTIME3D = CURTIME
            WDATE3D = CURDATE
            WTIME3D = CURTIME
            UPNAM3D = CMISS3
            
            DO  122  LVL = 1, NLAYS3D
                VGLVS3D( LVL ) = BADVAL3
122         CONTINUE
            DO  123  LVL = 1, MXDESC3
                FDESC3D( LVL ) = CMISS3
                UPDSC3D( LVL ) = CMISS3
123         CONTINUE
            DO  124  LVL = 1, MXVARS3
                UNITS3D( LVL ) = CMISS3
                VDESC3D( LVL ) = CMISS3
124         CONTINUE    

            DESC3 = .TRUE.
            RETURN

        END IF		!  if "buffered" file


C.......   Else read characteristics from the file header:

        CALL NCAGTC( FNUM, NCGLOBAL, 'EXEC_ID', EXECN3D, MXDLEN3, IERR)
        IF ( IERR .NE. 0 ) THEN
            EXECN3D = CMISS3
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute EXEC_ID ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGT( FNUM, NCGLOBAL, 'VGTYP', VGTYP3D, IERR )
        IF ( IERR .NE. 0 ) THEN
            VGTYP3D = IMISS3
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute VFTYP ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGT( FNUM, NCGLOBAL, 'VGTOP', VGTOP3D, IERR )
        IF ( IERR .NE. 0 ) THEN
            VGTOP3D = IMISS3
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute VFTOP ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGT( FNUM, NCGLOBAL, 'VGLVLS', VGLVS3D, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute VGLVS ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGT( FNUM, NCGLOBAL, 'CDATE', CDATE3D, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute CDATE ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGT( FNUM, NCGLOBAL, 'CTIME', CTIME3D, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute CTIME ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGT( FNUM, NCGLOBAL, 'WDATE', WDATE3D, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute WDATE ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGT( FNUM, NCGLOBAL, 'WTIME', WTIME3D, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute WTIME ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGTC( FNUM, NCGLOBAL, 'UPNAM', UPNAM3D, NAMLEN3, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute UPNAM ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGTC( FNUM, NCGLOBAL, 'FILEDESC', FDESC3D,
     &               MXDLEN3 * MXDESC3, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute FDESC ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed

        CALL NCAGTC( FNUM, NCGLOBAL, 'HISTORY', UPDSC3D,
     &               MXDLEN3 * MXDESC3, IERR )
        IF ( IERR .NE. 0 ) THEN
            WRITE( MESG, '( A, I9 )' ) 'netCDF error number', IERR
            CALL M3MSG2( MESG )
            WRITE( MESG, '( 4 A )' )
     &          'netCDF file header attribute UPDSC ',
     &          'not available for file "', FNAME, '"'
            CALL M3WARN( 'DESC3', 0, 0, MESG )
            DESC3 = .FALSE.
            RETURN
        END IF          !  ierr nonzero:  NCAGT() failed


        IF ( FTYPE3D .NE. DCTNRY3 ) THEN

            DO  222  VAR = 1, NVARS3D

                CALL NCAGTC( FNUM, VINDX3( VAR,FID ), 'units',
     &                       UNITS3D( VAR ), NAMLEN3, IERR )
                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '(A,I9)' )'netCDF error number', IERR
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( 4 A )' )
     &              'Error reading netCDF variable attribute "units"',
     &              'for file "', FNAME, '"'
                    CALL M3WARN( 'DESC3', 0, 0, MESG )
                    DESC3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

                CALL NCAGTC( FNUM, VINDX3( VAR,FID ), 'var_desc',
     &                       VDESC3D( VAR ), MXDLEN3, IERR )
                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '(A,I9)' )'netCDF error number', IERR
                    CALL M3MSG2( MESG )
                    WRITE( MESG, '( 4 A )' )
     &              'Error reading netCDF variable attribute ',
     &              '"var_desc" for file "', FNAME, '"'
                    CALL M3WARN( 'DESC3', 0, 0, MESG )
                    DESC3 = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCAGT() failed

222         CONTINUE

C...............   TSERIES3 files:  get numbers of extra per-variable 
C...............   attributes, lists of names and values for them:
 
            IF ( FTYPE3D .EQ. TSRIES3 ) THEN
 
                DO 244 VAR = 1,NVARS3D
              
                    CALL NCAGT( FNUM, VINDX3( VAR,FID ), 'natts',
     &                          NATTS3D( VAR ), IERR )
                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( A, I9)' )
     &                      'netCDF error number', IERR
                        CALL M3MSG2( MESG )
                        WRITE( MESG, '( 4 A )' )
     &                      'Error reading netCDF dimension ',
     &                      '"natts" for file "', FNAME, '"'
                        CALL M3WARN( 'DESC3', 0, 0, MESG )
                        DESC3 = .FALSE.
                        RETURN
                    END IF

                    DO 233 ATT = 1, NATTS3D( VAR )   ! loop on each addtnl atts
                   

                        CALL NCANAM( FNUM, VINDX3( VAR,FID ), ATT+4, 
     &                               ATNAM3D( ATT,VAR ), IERR )

                        IF ( IERR .NE. 0 ) THEN
                            WRITE( MESG, '( A, I9)' )
     &                      'netCDF error number', IERR
                            CALL M3MSG2( MESG )
                            WRITE( MESG, '( A, I9, 1X, 4A )' )
     &                      'Error reading attribute ', ATT,
     &                      ' for variable "',  VNAME3D( VAR ), 
     &                      '" in file "', FNAME, '"'
                            CALL M3WARN( 'DESC3', 0, 0, MESG )
                            DESC3 = .FALSE.
                            RETURN
                        END IF
                   
                        CALL NCAGT( FNUM, VINDX3( VAR,FID ), 
     &                              ATNAM3D( ATT,VAR ),
     &                              FATTS3D( ATT,VAR ), IERR )
                        IF ( IERR .NE. 0 ) THEN
                            WRITE( MESG, '( 7A )' )
     &                      'Error reading attribute "', 
     &                      ATNAM3D( ATT,VAR ),
     &                      '" for variable "',  VNAME3D( VAR ), 
     &                      '" in file "', FNAME, '"'
                            CALL M3WARN( 'DESC3', 0, 0, MESG )
                            DESC3 = .FALSE.
                            RETURN
                        END IF
                  
233                 CONTINUE		!  end loop on ATTs for this VAR
                  
244             CONTINUE	!  end loop on variables VAR in this file

            END IF		!  if ftype tseries3
 
        END IF          !  if not a dictionary file

        DESC3 = .TRUE.

        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91001   FORMAT ( //5X , '>>> WARNING in subroutine DESC3 <<<',
     &            /5X , 3A,
     &            /5X , A , I5, // )

91010   FORMAT ( //5X , '>>> WARNING in subroutine DESC3 <<<',
     &            3 ( /5X , A , : ) , I5, // )

91020   FORMAT ( //5X , '>>> WARNING in subroutine DESC3 <<<',
     &            /5X , A , I5, A, 
     &            /5X , A , I5 , // )

        END

