C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/flush3.f,v 1.2 2000/11/28 21:22:45 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION  FLUSH3 ( FNAME )

C***********************************************************************
C  function body starts at line  60
C
C  FUNCTION:  Flushes file with logical name FNAME
C
C  RETURN VALUE:  TRUE iff it succeeds.
C
C  PRECONDITIONS REQUIRED:  FNAME exists and has been opened
C
C  SUBROUTINES AND FUNCTIONS CALLED:  none
C
C  REVISION  HISTORY:  
C       prototype 8/95 by CJC
C
C       Modified  5/98 by CJC for OpenMP thread-safety
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME   !  logical name of file to be opened


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         INDEX1  !  look up names in tables
        INTEGER         TRIMLEN
        EXTERNAL        INDEX1, TRIMLEN


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         FILE            !  file index
        INTEGER         V               !  vble index
        INTEGER         IERR            !  netCDF error status return
        CHARACTER*16    FIL16           !  scratch file-name buffer
        CHARACTER*80    MSGBUF


C***********************************************************************
C   begin body of function  FLUSH3

        IF ( .NOT. FINIT3 ) THEN
            FLUSH3 = .TRUE.
            RETURN
        END IF

        IF ( LEN( FNAME ) .GT. 16 ) THEN
            MSGBUF = 'File name length bad for "' // FNAME // '"'
            CALL M3MSG2( MSGBUF )
            WRITE( MSGBUF, '(A, I9 )' )
     &          'Max length 16; actual:', LEN( FNAME )
            CALL M3MSG2( MSGBUF )
            FLUSH3 = .FALSE.
            RETURN
        END IF
        

C.......   Find netCDF index for the file, and check time step availability:

        FIL16 = FNAME   !  fixed-length-16 scratch copy of name
        FILE  = INDEX1( FIL16, COUNT3, FLIST3 )

        IF ( FILE .EQ. 0 ) THEN !  file not open.
            MSGBUF = 'File "' // FNAME( 1:TRIMLEN( FNAME ) ) //
     &               '" not currently open'
            CALL M3MSG2( MSGBUF )
            FLUSH3 = .FALSE.
            RETURN
        END IF

        IF( CDFID3( FILE ) .GE. 0 ) THEN
!$OMP       CRITICAL( S_NC )
            IERR = NF_SYNC( CDFID3( FILE ) )
!$OMP           END CRITICAL( S_NC )
            IF ( IERR .NE. 0 ) THEN
                MSGBUF =  'Error flushing netCDF file "' //
     &                    FNAME( 1:TRIMLEN( FNAME ) ) // '"'
                CALL M3MSG2( MSGBUF )
                WRITE( MSGBUF, '( A, I9 )' ) 
     &                  'netCDF error number', IERR
                CALL M3MSG2( MSGBUF )
                FLUSH3 = .FALSE.
                RETURN
            END IF      !  if ierr nonzero
        END IF          !  if cdfid(file) positive or not

        FLUSH3 = .TRUE.

        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91010   FORMAT ( //5X , '>>> WARNING in subroutine FLUSH3 <<<',
     &            3 ( /5X , A , : ) , I5, // )

91020   FORMAT ( //5X , '>>> WARNING in subroutine FLUSH3 <<<',
     &            3 ( /5X , A , : ) // )

C...........   Log message formats................... 92xxx

92010   FORMAT ( /5X , A, / )

        END

