
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/getnum.f,v 1.2 2000/11/28 21:22:47 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        INTEGER FUNCTION GETNUM ( LO , HI , DEFAULT , PROMPT )

C********************************************************************
C       function body starts at line  83
C
C       CALLS:      TRIMLEN, ENVYN, M3EXIT
C
C  FUNCTION:
C
C       Display the  PROMPT  for an integer between  LO  and  HI,
C       get the user's response and check that it is within range.
C       Return DEFAULT if the user hits <RET>.  Reprompts on error
C       for up to5 attempts.
C
C  REVISION HISTORY:
C
C       Created 3/89 by CJC.
C       Revised 5/90 by CJC:  error abort is via EXWST.
C       Revised 8/90 by CJC:  TEMP.LET; treat CONTROL-Z as exit signal.
C       Revised 2/93 by CJC:  CRAY version using DECODE.
C       Revised 8/96 by CJC:  treats "!" as a delimiter
C       Revised 1/97 by CJC:  logs result
C
C  ARGUMENT LIST DESCRIPTION:
C
C    Input arguments:
C
C        LO       Minimum allowed return value
C        HI       Maximum allowed return value
C        DEFAULT  Default return value
C        PROMPT   Prompt for user
C
C    Output arguments:  none
C
C  RETURNS   user response after checking its range; or default.
C
C********************************************************************

        IMPLICIT NONE

C.......   ARGUMENTS:

        INTEGER         LO , HI
        INTEGER         DEFAULT
        CHARACTER*(*)   PROMPT


C.......   EXTERNAL FUNCTIONS:

        LOGICAL         ENVYN
        INTEGER         TRIMLEN
        INTEGER         LBLANK          ! # of leading blanks
        EXTERNAL        ENVYN, TRIMLEN , LBLANK


C.......   LOCAL VARIABLES:

        INTEGER         J , M , N
        INTEGER         LLO , LHI , LDF
        INTEGER         ANSWER
        INTEGER         ERRCNT
        INTEGER         IOS
        CHARACTER*16    BUFFER , DEFSTR
        CHARACTER*8     FMTSTR

        LOGICAL         PROMPTON

        LOGICAL         FIRSTIME
        DATA            FIRSTIME / .TRUE. /
        SAVE            FIRSTIME, PROMPTON

C......................................................................
C       begin GETNUM

        IF( FIRSTIME ) THEN

            PROMPTON = ENVYN( 'PROMPTFLAG', 'Prompt for input flag',
     &                      .TRUE., IOS )
            FIRSTIME = .FALSE.
 
        END IF

        M  =  TRIMLEN( PROMPT )

        IF( .NOT. PROMPTON ) THEN
            GETNUM = DEFAULT
            WRITE( *,92310 ) 
     &          'Using default value', DEFAULT, 'for query',
     &          '"' , PROMPT ( 1:M ) , '"'
            RETURN
        END IF

        ERRCNT =  0

        LLO  =  MIN ( LO , HI )
        LHI  =  MAX ( LO , HI )
        LDF  =  MIN ( LHI , MAX ( LLO , DEFAULT ) )

        WRITE ( DEFSTR , 93200 ) LDF
        J  =  1 + LBLANK( DEFSTR )


100     CONTINUE
        WRITE ( *,93000 ) PROMPT ( 1:M ) , DEFSTR ( J:15 )

        BUFFER =  '           '
        READ ( *, 93100, ERR=900, IOSTAT=IOS, END=999 ) BUFFER

        N = INDEX( BUFFER, '!' )
        IF ( N .GT. 0 ) BUFFER( N : LEN( BUFFER ) ) = ' '

        IF ( BUFFER ( 1:1 )  .EQ. ' ' )  THEN
            GETNUM  =  LDF
            WRITE( *,92300 ) 'Using default', LDF
        ELSE

	    M = TRIMLEN( BUFFER )
            WRITE( FMTSTR, 94010 ) M
            READ( BUFFER, FMTSTR, IOSTAT=IOS, ERR=400 )  ANSWER

            IF ( ANSWER .LT. LLO  .OR.  ANSWER .GT. LHI )  THEN
                ERRCNT  =  ERRCNT + 1
                WRITE ( *,92100 ) ANSWER , LLO , LHI , ERRCNT
                IF ( ERRCNT .LT. 5 )  GO TO  100

                GO TO  500      !  max error count exceeded

            END IF

            GETNUM  =  ANSWER
            WRITE( *,92300 ) 'Using response', ANSWER

        END IF

        RETURN


400     CONTINUE        !  error in read from BUFFER

        ERRCNT  =  ERRCNT + 1
        WRITE ( *,92200 )  ERRCNT
        IF ( ERRCNT .LT. 5 )  GO TO  100


500     CONTINUE        !  more than 5 entry errors

        CALL M3EXIT( 'GETNUM', 0, 0, 'Maximum error count exceeded', 1 )


900     CONTINUE        !  error in read from terminal

        ERRCNT  =  ERRCNT + 1
        IF ( IOS .GT. 0 ) THEN
            IF ( ERRCNT .LT. 5 ) THEN
                WRITE ( *,92000 ) IOS , ERRCNT
                GO TO  100
            ELSE
                CALL M3EXIT( 'GETNUM', 0, 0, 
     &                       'Maximum error count exceeded', 1 )
            END IF
        END IF

999     CONTINUE        !  end-of-file condition

        CALL M3EXIT( 'GETNUM', 0, 0, 'END-OF-FILE encountered', 1 )


C................   end body of GETNUM  .......................................


92000   FORMAT ( /5X , '>>> ERROR IN ROUTINE GETNUM <<< ' ,
     &         //10X , 'Error reading response'               ,
     &         /10X  , 'I/O error status number = ' , I3                 ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'
     &         )

92100   FORMAT ( /5X , '>>>  RESPONSE ERROR  <<<'      ,
     &         //10X , 'Your response '           , I7 ,
     &                 ' not in the range '       , I7 ,
     &                 ' ... ' , I7 ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'            , /
     &         )

92200   FORMAT ( /5X , '>>>  RESPONSE ERROR  <<<'         ,
     &         //10X , 'Did not understand your response' ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'                , /
     &         )

92300   FORMAT ( /5X , A, I10 )

92310   FORMAT ( /5X , A, I10, 2X, A, /5X , 3A )

93000   FORMAT ( /5X , A , ' [' , A , '] >> ' , $ )

93100   FORMAT ( A15 )

93200   FORMAT ( I15 )

94010   FORMAT ( '(I', I3, ')' )

        END

C................   end   GETNUM  ....................................

