/**  
    Version "@(#)$Header$"
    EDSS/Models-3 I/O API.  Copyright (C) 1992-2002 MCNC
    Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
    See file "LGPL.txt" for conditions of use.
**/

/**************************************************************************
C PURPOSE:
C	returns list of attributes, their types, and sizes for the
C	file FNAME and variable VNAME (or ALLVAR3 for "global" file
C	attributes).
C       Wrapper around I/O API Fortran-binding routine INQATT3()
C
C PRECONDITIONS:
C	FNAME already opened by OPEN3() or open3c()
C	VNAME a valid variable in FNAME, or else is ALLVAR3=='ALL'
C
C CALLS:
C	Fortran I/O API's INQATT3()
C
C REVISION HISTORY:
C	Prototype 1/2002 by C
C	Cray conditional-compile bug fix 4/2002 by C
**************************************************************************/

#include  <string.h>
#include  "iodecl3.h"


		/** HACKS FOR FELDMAN-DESCENDED FC'S FOLLOW: **/

#if FLDMN

#define INQATT3 inqatt3_

#elif defined(__hpux) || defined(_AIX)

#define INQATT3 inqatt3

#endif

#define  BUFLEN  512


#if defined(INQATT3)

    extern int INQATT3( const char * fname ,
                        const char * vname ,
                        const int  * mxatts,
                        int        * natts,
                        char       * aname,
                        int          atypes[],
                        int          asizes[],
                        int          fnamelen ,
                        int          vnamelen ,
                        int          anamelen ) ;

int inqatt3c( const char          * fname ,
              const char          * vname ,
              int                   maxatt ,
              int                   attnamelen ,
              int                 * numatts ,
              char                * attname,
              int                   atype[],
              int                   asize[] )

    {       /*  begin body of inqatt3c() */

    return( INQATT3( fname ,
                     vname ,
                     & maxatt ,
                     numatts ,
                     attname ,
                     atype ,
                     asize ,
                     (int) strlen( fname ) ,
                     (int) strlen( vname ) ,
                     attnamelen ) ) ;

    }       /*  end body of  inqatt3c()  */


                	/** END  CASE OF FELDMAN-DESCENDED F77 TARGETS **/
                	/** NEXT CASE:  Win32 inqatt3c(): **/

#elif defined(_WIN32)

#include <stdio.h> /* for sprintf */

    extern int INQATT3( const char * fname ,
                        int          fnamelen ,
                        const char * vname ,
                        int          vnamelen ,
                        const int  * mxatts,
                        int        * natts,
                        char       * aname,
                        int          anamelen ,
                        int          atypes[],
                        int          asizes[] ) ;

int inqatt3c( const char          * fname ,
              const char          * vname ,
              int                   maxatt ,
              int                   attnamelen ,
              int                 * numatts ,
              char                * attname,
              int                   atype[],
              int                   asize[] )

    {       /*  begin body of  inqatt3c() */

    return( INQATT3( fname , (int) strlen( fname ) ,
                     vname , (int) strlen( vname ) ,
                     & maxatt ,
                     numatts ,
                     attname , attnamelen ,
                     atype ,
                     asize ) ) ;

    }       /*  end body of  inqatt3c()  */


                	/** END  CASE OF Win32 **/
                	/** NEXT CASE:  CRAY-TARGETED inqatt3c(): **/

#elif  defined(_CRAY)   /** treatment:  CRAY arryas of strings?? **/

#if( 0 )    /** if 0 **/

#include <fortran.h>

/** -------------------- Cray fstr2cstr() ----------------------------- **/

static void  fstr2cstr( const _fcd   source, 
                        char       * target, 
                        int          tlen )
    {
    char *ptr, *bound, ch ;
    int   slen, length ;

    slen = _fcdlen( source ) ;
    tlen-- ;

    length = ( slen < tlen ? slen : tlen ) ;
    ptr    = _fcdtocp( source ) ;
    for ( bound  = ptr + length ; ptr < bound ; target++ , ptr++ )
        {
        *target = *ptr ;
        }
    
    *target = '\0' ;
    
    }           /** END Cray fstr2cstr() **/


/** --------------------- Cray cstr2fstr() ---------------------------- **/

static void  cstr2fstr( const char * source,
                        _fcd         target )
    {
    char *bound ;
    char *ptr ;
    char  ch ;
    int   length ;

    length = _fcdlen ( target ) ;
    ptr    = _fcdtocp( target ) ;
    for (  bound  = ptr + length ;  ptr < bound ;  ptr++ , source++ )
        {
        if ( !( ch = *source ) ) break ;
        *ptr = ch ;
        }
    for ( ; ptr < bound ; ptr++ )
        {
        *ptr = ' ' ;
        }
    }           /** END Cray cstr2fstr() **/


    extern int INQATT3( const _fcd   fname ,
                        const _fcd   vname ,
                        const int  * mxatts,
                        int        * natts,
                        const _fcd   aname[],
                        int          atypes[],
                        int          asizes[] ) ;

int inqatt3c( const char          * fname ,
              const char          * vname ,
              int                   maxatt ,
              int                   attnamelen ,
              int                 * numatts ,
              char                * attname,
              int                   atype[],
              int                   asize[] )

    {       /*  begin body of  inqatt3c() */

    _fcd  file ;
    _fcd  vble ;
    _fcd  atts ;
    _fcd  att  ;
    long  stat ;
    char *cptr ;
    int      i ;
    
    
    
    atts = (fcd_[]) malloc( maxatt * BUFLEN ) ;
    if ( ! atts ) return ( _btol( 0 ) ) ;

    file = _cptofcd( (char *)fname, (int) strlen( fname ) ) ;
    vble = _cptofcd( (char *)vname, (int) strlen( vname ) ) ;

    stat = INQATT3( file,
                    vble,
                  & maxatt, 
                    numatts, 
                    atts, 
                    atype, 
                    asize ) ;

    if ( _btol( stat ) )
        {
        for ( i = 0, att = atts , cptr = attname ; 
              i < * numatts ; 
              i++, att++, cptr += attnamelen )
            {
            fstr2cstr( cptr, attnamelen ) ;
            } ;
        } ;
   
    return( _btol( stat ) ) ;

    }       /*  end body of  inqatt3c()  */

#endif              /** if 0 **/

#else

#error   "Error compiling inqatt3c():  unsupported architecture"

#endif              /** IF FELDMAN-DESCENDED "FC" TARGETED, 
                        OR IF Win32, OR IF CRAY **/



