
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/opnlog3.f,v 1.2 2000/11/28 21:22:59 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE OPNLOG3 ( FID , EQNAME, STATUS )

C***********************************************************************
C  function body starts at line  103
C
C  FUNCTION:  puts description of file at FID into the program log.
C
C  PRECONDITIONS REQUIRED:  Models-3 file with index FID must
C             have already opened by OPEN3().  To be called by
C             OPEN3().
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C
C  REVISION  HISTORY:
C       Prototype 3/92 by CJC
C       Modified  7/94 by CJC to return new coordinate-system and
C                             grid-description parameters
C       Modified  5/98 by CJC for OpenMP thread-safety
C
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        INTEGER         FID             !  subscript for STATE3 arrays
        CHARACTER*256   EQNAME          !  physical file name
        INTEGER         STATUS          !  file-opening status

                        
C...........   EXTERNAL FUNCTIONS:
        
        CHARACTER*24    DT2STR
        CHARACTER*10    HHMMSS
        INTEGER         LBLANK
        INTEGER         TRIMLEN

        EXTERNAL        DT2STR, HHMMSS, LBLANK, TRIMLEN
        
        
C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER        FNUM       ! netCDF file ID from NCCRE()
        INTEGER        IERR       ! netCDF error-result argument
        CHARACTER*24   TVALUE     ! for DT2STR()  result
        CHARACTER*10   SVALUE     ! for HHMMSS()  result
        CHARACTER*80   EXECID     ! execution-id
        CHARACTER*20   RBUF       ! for NROWS3D
        CHARACTER*20   CBUF       ! for NROWS3D
        CHARACTER*20   LBUF       ! for NCOLS3D
        CHARACTER*20   VBUF       ! for NVARS3D
        CHARACTER*20   TBUF       ! for NTHIK3D  
        CHARACTER*20   KBUF       ! for KF records
                                                 
        CHARACTER*16   SNAME( 0:6 )
        DATA           SNAME  
     &      / CMISS3,             !  0:  bad status
     &        'OLD:READ-ONLY', 	  !  1
     &        'OLD:READ-WRITE',   !  2
     &        'NEW(READ-WRITE)',  !  3
     &        'UNKNOWN(R-W)',	  !  4
     &        'NEW(CREAT/TRUNC)', !  5
     &        CMISS3 /	          !  6:  bad status

        CHARACTER*8    TYPNAME( -4 : 7 )
        DATA           TYPNAME
     &      / 'UNKNOWN',        !  -4:  file type error
     &        'KFEVNT3',        !  -3:  known file types
     &        'DGRAPH3',        !  -2:    "
     &        'CUSTOM3',        !  -1:    "
     &        'DCTNRY3',        !   0:    "
     &        'GRDDED3',        !   1:    "
     &        'BNDARY3',        !   2:    "
     &        'IDDATA3',        !   3:    "
     &        'PROFIL3',        !   4:    "
     &        'GRNEST3',        !   5:    "
     &        'SMATRX3',        !   6:    "
     &        'UNKNOWN'         !   7:    "
     &        /
        
        SAVE            SNAME, TYPNAME
        
C.............................................................................
C   begin body of subroutine  OPEN3

C.......   Log the file description:
        
        FNUM  = CDFID3( FID )
                
!$OMP   CRITICAL( S_LOGOUT )

C.......   Get execution-ID for the program which produced this file:

        IF( FNUM .GE. 0 ) THEN
            CALL NCAGTC( FNUM, NCGLOBAL, 
     &                   'EXEC_ID', EXECID, MXDLEN3, IERR )
        ELSE            !  "BUFFERED" files
            IERR   = 0
            EXECID = EXECN3
        END IF
        IF ( IERR .NE. 0 ) THEN
            EXECID = CMISS3
            WRITE( LOGDEV,91010 )
     &          'Warning netCDF file header attribute EXEC_ID.',
     &          'Not available for file:  ' // FLIST3( FID ) ,
     &          'netCDF error number', IERR
            WRITE( LOGDEV,93000 ) 
     &  ' ',
     &  '"' // FLIST3( FID )( 1:TRIMLEN( FLIST3( FID ) ) ) // 
     &  '" opened as ' // SNAME( MAX( MIN( STATUS, 6 ), 0  ) ), 
     &  'File name "'  // EQNAME( 1:TRIMLEN( EQNAME ) ) // '"',
     &  'File type '   // TYPNAME( MIN( MAX( FTYPE3( FID ), -4 ), 7 ) ),
     &  'Grid name "'  // GDNAM3( FID )( 1:TRIMLEN( GDNAM3( FID ) ) )
     &                 // '"'
        ELSE
            WRITE( LOGDEV,93000 ) 
     &  ' ',
     &  '"' // FLIST3( FID )( 1:TRIMLEN( FLIST3( FID ) ) ) // 
     &  '" opened as ' // SNAME( MAX( MIN( STATUS, 6 ), 0 ) ), 
     &  'File name "'  // EQNAME( 1:TRIMLEN( EQNAME ) ) // '"',
     &  'File type '   // TYPNAME( MIN( MAX( FTYPE3( FID ), -4 ), 7 ) ),
     &  'Execution ID "' // EXECID( 1:TRIMLEN( EXECID ) ) // '"',
     &  'Grid name "'  // GDNAM3( FID )( 1:TRIMLEN( GDNAM3( FID ) ) )
     &                 // '"'
        END IF          !  ierr nonzero:  NCAGT() failed
        
        
        WRITE( RBUF,94010 ) NROWS3( FID )
        WRITE( CBUF,94010 ) NCOLS3( FID )
        WRITE( LBUF,94010 ) NLAYS3( FID )
        WRITE( VBUF,94010 ) NVARS3( FID )
        IF ( FTYPE3( FID ) .EQ. BNDARY3 ) THEN
            WRITE( TBUF,94010 ) NTHIK3( FID )
            WRITE( LOGDEV,93000 ) 
     &          'Dimensions: '
     &          // RBUF( LBLANK( RBUF )+1 : 20 ) // ' rows, '
     &          // CBUF( LBLANK( CBUF )+1 : 20 ) // ' cols, '
     &          // LBUF( LBLANK( LBUF )+1 : 20 ) // ' lays, '
     &          // VBUF( LBLANK( VBUF )+1 : 20 ) // ' vbles, '
     &          // TBUF( LBLANK( TBUF )+1 : 20 ) // ' cells thick'
        ELSE IF ( FTYPE3( FID ) .EQ. KFEVNT3 ) THEN
            WRITE( TBUF,94010 ) NTHIK3( FID )
            WRITE( KBUF,94010 ) MXREC3( FID )
            WRITE( LOGDEV,93000 ) 
     &          'Dimensions: '
     &          // RBUF( LBLANK( RBUF )+1 : 20 ) // ' rows, '
     &          // CBUF( LBLANK( CBUF )+1 : 20 ) // ' cols, '
     &          // LBUF( LBLANK( LBUF )+1 : 20 ) // ' lays, '
     &          // VBUF( LBLANK( VBUF )+1 : 20 ) // ' vbles, '
     &          // TBUF( LBLANK( TBUF )+1 : 20 ) // ' max recs/cell, '
     &          // KBUF( LBLANK( KBUF )+1 : 20 ) // ' total recs'
        ELSE
            WRITE( LOGDEV,93000 ) 
     &          'Dimensions: '
     &          // RBUF( LBLANK( RBUF )+1 : 20 ) // ' rows, '
     &          // CBUF( LBLANK( CBUF )+1 : 20 ) // ' cols, '
     &          // LBUF( LBLANK( LBUF )+1 : 20 ) // ' lays, '
     &          // VBUF( LBLANK( VBUF )+1 : 20 ) // ' vbles'
        END IF
        
       WRITE( LOGDEV,93020 ) 'NetCDF ID: ', CDFID3( FID )

       IF( TSTEP3( FID ) .EQ. 0 ) THEN
            WRITE( LOGDEV,93000 ) 'Time-independent data.'
        ELSE
            TVALUE = DT2STR( SDATE3( FID ), STIME3( FID ) ) 
            SVALUE = HHMMSS( ABS( TSTEP3( FID ) ) ) 
            WRITE( LOGDEV,93020 ) 
     &      'Starting date and time', SDATE3( FID ),  STIME3( FID ),
     &      ' (' // TVALUE( 1: TRIMLEN( TVALUE ) ) //  ')',
     &      'Timestep                     ', TSTEP3( FID ),
     &      ' (' // SVALUE( 1: TRIMLEN( SVALUE ) ) //  ' hh:mm:ss)',
     &      'Maximum current record number', MXREC3( FID )
                
        END IF          !  time step zero or not

	CALL FLUSH( LOGDEV )
        
!$OMP   END CRITICAL( S_LOGOUT )


        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91010   FORMAT ( //5X , '>>> WARNING in subroutine OPNLOG3 <<<',
     &            3 ( /5X , A , : ) , I5, // )


C...........   Informational log message formats..... 93xxx

93000   FORMAT ( 5X, A )

93020   FORMAT ( 5X, A, I9, ':', I6.6, A, /5X, A, I9.6, A,
     &          /5X, A, I8 )

C...........   Internal buffering formats............ 94xxx

94010   FORMAT( I20 )

        END

