/**  
    Version "@(#)$Header$"
    EDSS/Models-3 I/O API.  Copyright (C) 1992-2002 MCNC
    Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
    See file "LGPL.txt" for conditions of use.
**/

/**************************************************************************
C PURPOSE:
C	 Reads the attribute named ANAME for the variable VNAME in the
C	file FNAME into AVAL[].  If VNAME == ALLVAR3, reads global
C	attribute ANAME.
C       Wrapper around I/O API Fortran-binding routines RDATT3()
C       for numeric attributes, and RDATTC() for character string
C       attributes)
C
C PRECONDITIONS:
C	FNAME already opened by OPEN3() or open3c()
C	VNAME a valid variable in FNAME, or else is ALLVAR3=='ALL'
C
C CALLS:
C	Fortran I/O API's RDATT3(), RDATTC()
C
C REVISION HISTORY:
C	Prototype 1/2002 by C
**************************************************************************/

#include  <string.h>
#include  "iodecl3.h"


		/** HACKS FOR FELDMAN-DESCENDED FC'S FOLLOW: **/

#if FLDMN

#define RDATT3 rdatt3_
#define RDATTC rdattc_

#elif defined(__hpux) || defined(_AIX)

#define RDATT3 rdatt3
#define RDATTC rdattc

#endif


#if defined(RDATT3)

    extern int RDATT3( const char * fname ,
                       const char * vname ,
                       const char * aname ,
                       const int  * atype,
                       const int  * amax,
                       int        * asize,
                       void       * aval ,
                       int          fnamelen, 
                       int          vnamelen, 
                       int          anamelen ) ;

    extern int RDATTC( const char * fname ,
                       const char * vname ,
                       const char * aname ,
                       char       * aval ,
                       int          fnamelen, 
                       int          vnamelen, 
                       int          anamelen, 
                       int          avallen ) ;

int rdatt3c( const char          * fname ,
             const char          * vname ,
             const char          * aname ,
             int                   atyp ,
             int                   amax ,
             int                 * asiz ,
             void                * att )

    {       /*  begin body of rdatt3c() */

    if ( atyp == M3CHAR )
        {
        return( RDATTC( fname ,
                        vname ,
                        aname ,
                        att
                        (int) strlen( fname ) ,
                        (int) strlen( vname ) ,
                        (int) strlen( aname ) ,
                        amax ) ;
        } ;
    else
        {
        return( RDATT3( fname ,
                        vname ,
                        aname ,
                      & atyp ,
                      & amax ,
                        asiz ,
                        att
                        (int) strlen( fname ) ,
                        (int) strlen( vname ) ,
                        (int) strlen( aname ) ) ;
        }

    }       /*  end body of  rdatt3c()  */


                	/** END  CASE OF FELDMAN-DESCENDED F77 TARGETS **/
                	/** NEXT CASE:  Win32 rdatt3c(): **/

#elif defined(_WIN32)

#include <stdio.h> /* for sprintf */

    extern int RDATT3( const char * fname , fnamelen ,
                       const char * vname , vnamelen ,
                       const char * aname , anamelen ,
                       const int  * atype,
                       const int  * amax,
                       int        * asize,
                       void       * aval ) ;

    extern int RDATTC( const char * fname , fnamelen ,
                       const char * vname , vnamelen ,
                       const char * aname , anamelen ,
                       char       * aval  , avallen ) ;


int rdatt3c( const char          * fname ,
             const char          * vname ,
             const char          * aname ,
             int                   atyp ,
             int                   amax ,
             int                 * asiz ,
             void                * att )

    {       /*  begin body of  rdatt3c() */
    if ( atyp == M3CHAR )
        {
        return( RDATTC( fname , (int) strlen( fname ) ,
                        vname , (int) strlen( vname ) ,
                        aname , (int) strlen( aname ) ,
                        att   , amax ) ;
        } ;
    else{
        return( RDATT3( fname , (int) strlen( fname ) ,
                        vname , (int) strlen( vname ) ,
                        aname , (int) strlen( aname ) ,
                      & atyp ,
                      & amax ,
                        asiz ,
                        att ) ;
        }
    }       /*  end body of  rdatt3c()  */


                	/** END  CASE OF Win32 **/
                	/** NEXT CASE:  CRAY-TARGETED rdatt3c(): **/

#elif  defined(_CRAY)   /** treatment:  CRAY arryas of strings?? **/


#include <fortran.h>

    extern int RDATT3( const _fcd   fname ,
                       const _fcd   vname ,
                       const _fcd   aname ,
                       const int  * atype,
                       const int  * amax,
                       int        * asize,
                       void       * aval ) ;

    extern int RDATTC( const _fcd   fname ,
                       const _fcd   vname ,
                       const _fcd   aname ,
                       _fcd         aval ) ;

int rdatt3c( const char          * fname ,
             const char          * vname ,
             const char          * aname ,
             int                   atyp ,
             int                   amax ,
             int                 * asiz ,
             void                * att )

    {       /*  begin body of  rdatt3c() */

    _fcd  file ;
    _fcd  vble ;
    _fcd  atnm ;
    _fcd  aval ;

    file = _cptofcd( (char *)fname, (int) strlen( fname ) ) ;
    vble = _cptofcd( (char *)vname, (int) strlen( vname ) ) ;
    atnm = _cptofcd( (char *)aname, (int) strlen( aname ) ) ;

    if ( atyp == M3CHAR )
        {
        aval = _cptofcd( (char *)att, amax ) ;
        return( _btol( RDATT3( fname , (int) strlen( fname ) ,
                               vname , (int) strlen( vname ) ,
                               aname , (int) strlen( aname ) ,
                             & atyp ,
                             & amax ,
                               asiz ,
                               att ) ) ;
        } ;
    else{
        return( _btol( RDATTC( fname , (int) strlen( fname ) ,
                               vname , (int) strlen( vname ) ,
                               aname , (int) strlen( aname ) ,
                               att   , amax ) ) ;
        }

    }       /*  end body of  rdatt3c()  */

#else

#error   "Error compiling rdatt3c():  unsupported architecture"

#endif              /** IF FELDMAN-DESCENDED "FC" TARGETED, 
                        OR IF Win32, OR IF CRAY **/



