
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/rdvars.F,v 1.3 2000/11/28 21:13:42 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION RDVARS( FID, VID, DIMS, DELTS, DELTA, BUFFER )

C***********************************************************************
C  function body starts at line  81
C
C  FUNCTION:  reads "variables" part of time step records from Models-3
C             files with index FID, for variable with index VID for
C             routines RDCUSTOM, RDGRDDED, RDBNDARY, RDIDDATA, RDPROFIL,
C             and RDGRNEST.
C             If VID is -1 = ALLAYS3 reads all variables; if LAYER is -1,
C             reads all layers.
C
C  MACHINE DEPENDENCY:
C	TYPSIZE( 6 ) must be sizeof( double )/sizeof( real )
C
C  RETURN VALUE:  TRUE iff the operation succeeds (and the data is available)
C
C  PRECONDITIONS REQUIRED:  Should only be called by the above routines,
C             after OPEN3() has checked for file and time step availability,
C             and after the above routines have set up DIMS, DELTS, and DELTA.
C
C  SUBROUTINES AND FUNCTIONS CALLED:  INDEX1
C
C  REVISION  HISTORY:  
C	prototype 3/92 by CJC
C
C	Modified  9/94 by CJC:  argument VID instead of VNAME
C
C       Modified  5/98 by CJC for OpenMP thread-safety
C
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        INTEGER         FID             !  file index within the STATE3 commons
        INTEGER         VID             !  variable index or -1 == ALL
        INTEGER         DIMS ( 5 )      !  corner arg array for NCVGT()
        INTEGER         DELTS( 5 )      !  corner arg array for NCVGT()
        INTEGER         DELTA           !  d(INDX) / d(NCVGTcall)
        REAL            BUFFER(*)       !  buffer array for input


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         IERR            !  netCDF error status return
        INTEGER         VAR             !  loop counter for file variables
        INTEGER         INDX            !  subscript location in BUFFER(*)
        CHARACTER*256   MESG
        
        INTEGER		TYPSIZE( 6 )    !  sizeof( variable ) / sizeof( real )
#if _CRAY || REAL8
        DATA            TYPSIZE / 1, 1, 1, 1, 1, 1 /
#endif
#if ! ( _CRAY || REAL8 )
        DATA            TYPSIZE / 1, 1, 1, 1, 1, 2 /
#endif


C***********************************************************************
C   begin body of function  RDVARS

C...........   Perform the reads, according to VID

        IF ( VID .EQ. ALLAYS3 ) THEN

            INDX = 1    !  starting subscript for BUFFER(*)

            DO  12  VAR = 1 , NVARS3( FID )

!$OMP           CRITICAL( S_NC )
                CALL NCVGT( CDFID3( FID ), VINDX3( VAR,FID ),
     &                      DIMS, DELTS, BUFFER( INDX ), IERR )
!$OMP           END CRITICAL( S_NC )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG,91010 )
     &                  'netCDF error number', IERR,
     &                  'Error reading variable ' // VLIST3( VAR,FID )
     &                  // 'from file ' // FLIST3( FID )
                    CALL M3MSG2( MESG )
                    RDVARS = .FALSE.
                    RETURN
                END IF          !  ierr nonzero:  NCVGTC() failed

                INDX = INDX  +  DELTA * TYPSIZE( VTYPE3( VAR,FID ) )

12          CONTINUE    !  end loop on variables VAR

        ELSE    !  read a specific variable

!$OMP       CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ), VINDX3( VID,FID ),
     &                  DIMS, DELTS, BUFFER, IERR )
!$OMP       END CRITICAL( S_NC )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG,91010 )
     &              'netCDF error number', IERR,
     &              'Error reading variable ' // VLIST3( VID,FID )
     &              // 'from file ' // FLIST3( FID )
                    CALL M3MSG2( MESG )
                RDVARS = .FALSE.
                RETURN
            END IF          !  ierr nonzero:  NCVGTC() failed, or succeeded

        END IF  !  read all variables, or read a specific variable

        RDVARS = .TRUE.
        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91010   FORMAT ( 3 ( A , :, 2X ) , I5, // )


        END

