
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/read4d.F,v 1.3 2000/11/28 21:13:43 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

      LOGICAL FUNCTION READ4D( FNAME, VNAME, LAYER, 
     &                         SDATE, STIME, TSTEP, NRECS, BUFFER )

C***********************************************************************
C  function body starts at line  128
C
C  FUNCTION:  
C       reads all the data for the specified LAYER and the timestep
C       sequence JDATE:JTIME:TSTEP:NRECS (formatted YYYYDDD and HHMMSS) 
C       from the disk-resident time-stepped I/O API data file with 
C       logical name FNAME, and stores it in BUFFER.
C       FNAME must be of type GRDDED3, BNDARY3, CUSTOM3, or TSERIES3.       
C       If LAYER is ALLAYS3 = -1, reads all layers.
C       Does NOT support VNAME = ALLVARS3.
C
C  RETURN VALUE:  
C       TRUE iff the entire operation succeeds (and the data is available)
C
C  PRECONDITIONS REQUIRED:  
C       FNAME is a disk-resident I/O API data file already opened by OPEN3()
C       FNAME has type GRDDED3, BNDARY3, CUSTOM3, or TSERIES3
C       FNAME is time-stepped, and its variable VNAME has been written
C       to FNAME for all the timesteps in the requested sequence.
C       Storage order for BUFFER is according to Fortran storage order
C       for subscript tuple  ( ..., [ LAYERS, ] TIMESTEPS )
C
C  REVISION  HISTORY:  
C       Adapted   8/1999 by CJC from READ3
C
C       Modified  1/2002 by CJC:  check TRIMLEN() of FNAME
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME           !  logical file name
        CHARACTER*(*)   VNAME           !  variable name, or 'ALL'
        INTEGER         LAYER           !  layer number, or 0
        INTEGER         SDATE           !  starting date, formatted YYYYDDD
        INTEGER         STIME           !  starting time, formatted HHMMSS
        INTEGER         TSTEP           !  time step,     formatted HHMMSS
        INTEGER         NRECS           !  number of time steps
        REAL            BUFFER(*)       !  input buffer array


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         INIT3      !  initialize I/O API
        INTEGER         INDEX1     !  look up names in name tables
        INTEGER         JSTEP3     !  compute time step record numbers
        LOGICAL         RDVARS     !  read time step records
        INTEGER         TRIMLEN    !  trimmed string length

        EXTERNAL        INIT3, INDEX1, JSTEP3, RDVARS, TRIMLEN
        EXTERNAL        INITBLK3   !  block data: initialize I/O state


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         FID             !  subscript  for STATE3 arrays
        INTEGER         VID, VAR, V     !  subscripts for STATE3 arrays
        INTEGER         STEP            !  time step record number
        INTEGER         JDATE           !  date, formatted YYYYDDD
        INTEGER         JTIME           !  time, formatted HHMMSS
        INTEGER         IERR            !  netCDF error status return
        INTEGER         IDUM            !  holds return value for INIT3()
        INTEGER         ITYP            !  holds file type
        INTEGER         DIMT( 5 )       !  corner   for NCVGT()
        INTEGER         DELT( 5 )       !  diagonal for NCVGT()
        INTEGER         TFLAG( 2 )      !  values array from NCVGT()
        INTEGER         DELTA           !  d(INDX) / d(NCVGTcall)
        INTEGER         DIMS ( 5 )      !  start  arg array for NCVPT()
        INTEGER         DELTS( 5 )      !  corner arg array for NCVPT()
        INTEGER         IREC            !  record number:  loop counter
        INTEGER         IT              !  subscript for STEP
        INTEGER         IS              !  subscript for BUFFER
        INTEGER         IL0, IL1        !  layer extent
        CHARACTER*16    FIL16           !  scratch file-name buffer
        CHARACTER*16    VAR16           !  scratch vble-name buffer
        CHARACTER*256   MESG
        LOGICAL         EFLAG

C...........   STATIC Variables:

        CHARACTER*7   TYPNAMES( -3 : 7 )
        DATA          TYPNAMES
     &      / 'UNKNOWN',        !  -3:  file type error
     &        'DGRAPH3',        !  -2:  known file types
     &        'CUSTOM3',        !  -1:    "
     &        'DCTNRY3',        !   0:    "
     &        'GRDDED3',        !   1:    "
     &        'BNDARY3',        !   2:    "
     &        'IDDATA3',        !   3:    "
     &        'PROFIL3',        !   4:    "
     &        'GRNEST3',        !   5:    "
     &        'SMATRX3',        !   6:    "
     &        'UNKNOWN' /       !   7:    "

        INTEGER       TYPSIZE( 6 )

#if _CRAY || REAL8
        DATA            TYPSIZE / 1, 1, 1, 1, 1, 1 /
#endif
#if ! ( _CRAY || REAL8 )
        DATA            TYPSIZE / 1, 1, 1, 1, 1, 2 /
#endif

C***********************************************************************
C   begin body of function  READ4D
C.......   Check that Models-3 I/O has been initialized:
 
        EFLAG = .FALSE.
!$OMP   CRITICAL( S_LOGDEV )
        IF ( .NOT. FINIT3 ) THEN
            LOGDEV = INIT3()
            READ4D = .FALSE.
            EFLAG = .TRUE.
        END IF          !  if not FINIT3
!$OMP   END CRITICAL( S_LOGDEV )
        IF ( EFLAG ) THEN
            CALL M3MSG2(  'READ4D:  I/O API not yet initialized.' )
            RETURN
        END IF

C...........   Check length of name arguments; copy into length=16 buffers
            
        EFLAG = .FALSE.
        IF ( TRIMLEN( FNAME ) .GT. NAMLEN3 ) THEN
            EFLAG = .TRUE.
            MESG  = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A , I10 )' )
     &          'Max file name length 16; actual:', TRIMLEN( FNAME )
            CALL M3MSG2( MESG )
        END IF          !  if len( fname ) > 16

        IF ( TRIMLEN( VNAME ) .GT. NAMLEN3 ) THEN
            EFLAG = .TRUE.
            MESG  = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I10 )'  )
     &          'Max vble name length 16; actual:', TRIMLEN( VNAME )
            CALL M3MSG2( MESG )
        END IF          !  if len( vname ) > 16
        
        IF ( EFLAG ) THEN
            MESG = 'Invalid variable or file name arguments'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
	    READ4D = .FALSE.
            RETURN
        END IF          !  if len( fname ) > 16, or if len( vname ) > 16

        FIL16 = FNAME
        VAR16 = VNAME
        

C.......   Find netCDF index for the file, and check time step availability:

        FIL16 = FNAME   !  fixed-length-16 scratch copy of name
        FID   = INDEX1( FIL16, COUNT3, FLIST3 )

        IF ( FID .EQ. 0 ) THEN  !  file not available

            MESG = 'File:  '//FIL16// ' not yet opened.'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN

        ELSE IF ( CDFID3( FID ) .EQ. BUFFIL3 ) THEN     !  BUFFERED "file"
        
            MESG = 'File:  '//FIL16// ' is BUFFERED.'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN

        ELSE IF ( CDFID3( FID ) .EQ. VIRFIL3 ) THEN

            MESG = 'File:  '//FIL16// ' is VIRTUAL.'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN

        ELSE IF ( TSTEP3( FID ) .EQ. 0 ) THEN

            MESG = 'File:  '//FIL16// ' is time-independent.'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN

        ELSE IF ( TSTEP3( FID ) .LT. 0 ) THEN

            MESG = 'File:  '//FIL16// ' is RESTART, not time-stepped.'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN

        ELSE IF ( VOLAT3( FID ) ) THEN     !  volatile file:  synch with disk

!$OMP CRITICAL( S_NC )
            CALL NCSNC( CDFID3( FID ), IERR )
!$OMP END CRITICAL( S_NC )
            IF ( IERR .NE. 0 ) THEN

                WRITE( MESG,91010 )
     &              'netCDF error number', IERR,
     &              'Error with disk synchronization for file:  '
     &              // FIL16
                CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
                READ4D = .FALSE.
                RETURN

            END IF      !  if ncsnc() failed

        END IF          !  if file not available, or if file is volatile


C.......   Check availability of requested  layer, variable:

        VAR16 = VNAME   !  fixed-length-16 scratch copy of name

        VID = INDEX1 ( VAR16, NVARS3( FID ), VLIST3( 1,FID ) )

        IF ( VID .EQ. 0 ) THEN
             MESG = 'File ' // FLIST3( FID )//' contains variables'
             CALL M3MSG2( MESG )
             DO  VAR=1,NVARS3( FID ), 3
                WRITE( MESG,91040 )
     &          (VLIST3( V,FID ),V=VAR,MIN( VAR+2, NVARS3( FID ) ) )
                CALL M3MSG2( MESG )
             END DO
             MESG = 'Requested variable "' //  VAR16 // 
     &              '" not available'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN
        END IF          !  end check on vid

        IF ( LAYER .EQ. ALLAYS3 ) THEN

            IL0 = 1
            IL1 = NLAYS3( FID )

        ELSE IF ( LAYER .GE. 1  .AND.  LAYER .LE. NLAYS3( FID ) ) THEN

            IL0 = LAYER
            IL1 = 1
        ELSE

            WRITE( MESG,91010 ) 'Requested layer:', LAYER
            CALL M3MSG2( MESG )
	    WRITE( MESG,91010 )
     &          'Layers in the file:  1 ...', NLAYS3( FID )
            CALL M3MSG2( MESG )
            MESG = 'LAYER not available in file ' // FIL16
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN

        END IF          !  end check on layer-number

C.......   Set up DIMS, DELS for RDVARS():

        ITYP = FTYPE3( FID )
        IF ( ITYP .EQ. CUSTOM3 ) THEN

            DIMS ( 1 ) = 1
            DELTS( 1 ) = NCOLS3( FID )

            DIMS ( 2 ) = IL0
            DELTS( 2 ) = IL1

            DELTS( 3 ) = 1
            IT = 3

            DELTA = NCOLS3( FID ) * NLAYS3( FID )

        ELSE IF ( ITYP .EQ. GRDDED3 ) THEN

            DIMS ( 1 ) = 1
            DELTS( 1 ) = NCOLS3( FID )

            DIMS ( 2 ) = 1
            DELTS( 2 ) = NROWS3( FID )

            DIMS ( 3 ) = IL0
            DELTS( 3 ) = IL1

            DELTS( 4 ) = 1
            IT = 4

            DELTA = NCOLS3( FID ) * NROWS3( FID ) * NLAYS3( FID )

        ELSE IF ( ITYP .EQ. BNDARY3 ) THEN

            DELTA = 2 * NTHIK3( FID )
            DELTA = DELTA * ( NCOLS3( FID ) + NROWS3( FID ) + DELTA )

            DIMS ( 1 ) = 1
            DELTS( 1 ) = DELTA

            DIMS ( 2 ) = IL0
            DELTS( 2 ) = IL1

            DELTS( 3 ) = 1

            DELTA = DELTA * NLAYS3( FID )

        ELSE IF ( ITYP .EQ. TSRIES3 ) THEN
 
            DIMS ( 1 ) = 1
            DELTS( 1 ) = NCOLS3( FID )

            DIMS ( 2 ) = 1
            DELTS( 2 ) = NROWS3( FID )

            DIMS ( 3 ) = IL0
            DELTS( 3 ) = IL1

            DELTS( 4 ) = 1
            IT = 4

            DELTA = NCOLS3( FID ) * NROWS3( FID ) * NLAYS3( FID )
 
        ELSE    !  unsupported file type:

            IDUM = MIN( MAX( ITYP, -3 ), 7 )
            WRITE( MESG, '( A, I7, 1X, 3A )' ) 
     &      'File type=', ITYP, '(', TYPNAMES( IDUM ), ')'
            CALL M3MSG2( MESG )
            MESG = 'Filetype not CUSTOM3, GRDDED3, BNDARY3, or TSRIES3'
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )

            READ4D = .FALSE.
            RETURN

        END IF          !  if file type is ...


C...........   Compute record number, and check availability:

        JDATE = SDATE
        JTIME = STIME

        STEP = JSTEP3( SDATE, STIME,
     &                 SDATE3( FID ),
     &                 STIME3( FID ),
     &                 TSTEP3( FID ) )
        
        IF ( STEP .LT. 0 ) THEN

            MESG = 'Time step error reading file:  ' // FIL16
            CALL M3MSG2( MESG )
            WRITE( MESG,91020 )
     &          'Requested date & time:    ', JDATE, JTIME
            CALL M3MSG2( MESG )
            WRITE( MESG,91020 )
     &          'File starting date & time:',
     &          SDATE3( FID ), STIME3( FID )
            CALL M3MSG2( MESG )
            WRITE( MESG,91030 )
     &          'File time step:           ', TSTEP3( FID )
            CALL M3MSG2( MESG )
            MESG = 'Time step error reading file:  ' // FIL16
            CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
            READ4D = .FALSE.
            RETURN
            
        END IF          !  check on step number

        DIMT( 1 ) = 1           !  field:  date or time
        DELT( 1 ) = 2           !  extent:  entire field
        DELT( 3 ) = 1           !  extent in timestep dimension
        
        DIMT( 2 ) = VID         !  variable-number
        DELT( 2 ) = 1           !  extent:  one variable
        
C...........   Compute record number, and check availability:

        IS    = 1
        JDATE = SDATE
        JTIME = STIME

C...........   Process the requested time step sequence:

        DO  IREC = 1, NRECS
        
            STEP = JSTEP3( JDATE,  JTIME,
     &                     SDATE3( FID ), STIME3( FID ),
     &                     TSTEP3( FID ) )

            DIMT( 3  ) = STEP        !  timestep dimension
            DIMS( IT ) = STEP

!$OMP       CRITICAL( S_NC )
                CALL NCVGT( CDFID3( FID ), TINDX3( FID ), 
     &                      DIMT, DELT, TFLAG, IERR )
!$OMP       END CRITICAL( S_NC )

            IF ( IERR .EQ. 8 ) THEN     !  timestep flag not yet written
               
                WRITE( MESG, '( 5A, I7.7, A, I6.6 )' ) 
     &              'Variable "', VNAME,  
     &              '" not yet written to file "', FNAME,  
     &              '" for date&time ', JDATE, ':', JTIME
                CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
                READ4D = .FALSE.
                RETURN
                        
            ELSE IF ( IERR .NE. 0 ) THEN
               
                WRITE( MESG,91010 ) 'netCDF error number', IERR
                CALL M3MSG2( MESG )
                MESG = 'Error reading netCDF time step flag for ' //
     &                 VLIST3( VID,FID ) // ' from ' // FIL16
                CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
                READ4D = .FALSE.
                RETURN
               
            ELSE IF( TFLAG( 1 ) .NE. JDATE .OR.
     &               TFLAG( 2 ) .NE. JTIME ) THEN
               
                WRITE( MESG, '( 5A, I7.7, A, I6.6 )' ) 
     &              'Variable "', VNAME,  
     &              '" not available in file "', FNAME,  
     &              '" for date&time ', JDATE, ':', JTIME
                CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
                READ4D = .FALSE.
                RETURN
               
            END IF          !  if ierr bad or if timestep flags bad
                   
            IF ( .NOT. RDVARS( FID, VID,   
     &                         DIMS, DELTS, DELTA,  
     &                         BUFFER( IS ) ) ) THEN

                WRITE( MESG, '( 5A, I7.7, A, I6.6 )' ) 
     &              'Error reading "', VNAME, '" from "', FNAME,  
     &              '" for date&time ', JDATE, ':', JTIME
                CALL M3WARN( 'READ4D', JDATE, JTIME, MESG )
                READ4D = .FALSE.
                RETURN

            END IF              ! if rdvars(0 failed

            !!  update state variable MXREC3D, and
            !!  increment time-flag and buffer-pointer:
            
            MXREC3( FID ) = MAX( MXREC3( FID ), ABS( STEP ) )
            IS = IS + DELTA * TYPSIZE( VTYPE3( VID,FID ) )
            CALL NEXTIME( JDATE, JTIME, TSTEP )
        
        END DO          !  end loop on output records.


        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91000   FORMAT ( 5A )

91010   FORMAT ( 3 ( A , :, I5, :, 2X ) )

91020   FORMAT ( A , I9, ':' , I6.6, :, A )

91030   FORMAT ( A , I6.6 )

91040   FORMAT ( 3 ( '"', A16, '"', :, 4X ) )

        END

