/**  
    Version "@(#)$Header$"
    EDSS/Models-3 I/O API.  Copyright (C) 1992-2002 MCNC
    Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
    See file "LGPL.txt" for conditions of use.
**/

/**************************************************************************
C PURPOSE:
C	Writes the attribute named ANAME for the variable VNAME to the
C	file FNAME.  If VNAME == ALLVAR3, writes global attribute ANAME.
C       Wrapper around I/O API Fortran-binding routines WRATT3()
C       for numeric attributes, and WRATTC() for character string
C       attributes)
C
C PRECONDITIONS:
C	FNAME already opened by OPEN3() or open3c()
C	VNAME a valid variable in FNAME, or else is ALLVAR3=='ALL'
C
C CALLS:
C	Fortran I/O API's WRATT3(), WRATTC()
C
C REVISION HISTORY:
C	Prototype 1/2002 by C
**************************************************************************/

#include  <string.h>
#include  "iodecl3.h"


		/** HACKS FOR FELDMAN-DESCENDED FC'S FOLLOW: **/

#if FLDMN

#define WRATT3 wratt3_
#define WRATTC wrattc_

#elif defined(__hpux) || defined(_AIX)

#define WRATT3 wratt3
#define WRATTC wrattc

#endif


#if defined(WRATT3)

    extern int WRATT3( const char * fname ,
                       const char * vname ,
                       const char * aname ,
                       const int  * atype,
                       const int  * amax,
                       void       * aval ,
                       int          fnamelen, 
                       int          vnamelen, 
                       int          anamelen ) ;

    extern int WRATTC( const char * fname ,
                       const char * vname ,
                       const char * aname ,
                       char       * aval ,
                       int          fnamelen, 
                       int          vnamelen, 
                       int          anamelen, 
                       int          avallen ) ;

int wratt3c( const char          * fname ,
             const char          * vname ,
             const char          * aname ,
             int                   atyp ,
             int                   amax ,
             int                 * asiz ,
             void                * att )

    {       /*  begin body of wratt3c() */

    if ( atyp == M3CHAR )
        {
        return( WRATTC( fname ,
                        vname ,
                        aname ,
                        att
                        (int) strlen( fname ) ,
                        (int) strlen( vname ) ,
                        (int) strlen( aname ) ,
                        amax ) ;
        } ;
    else
        {
        return( WRATT3( fname ,
                        vname ,
                        aname ,
                      & atyp ,
                      & amax ,
                        att
                        (int) strlen( fname ) ,
                        (int) strlen( vname ) ,
                        (int) strlen( aname ) ) ;
        }

    }       /*  end body of  wratt3c()  */


                	/** END  CASE OF FELDMAN-DESCENDED F77 TARGETS **/
                	/** NEXT CASE:  Win32 wratt3c(): **/

#elif defined(_WIN32)

#include <stdio.h> /* for sprintf */

    extern int WRATT3( const char * fname , fnamelen ,
                       const char * vname , vnamelen ,
                       const char * aname , anamelen ,
                       const int  * atype,
                       const int  * amax,
                       void       * aval ) ;

    extern int WRATTC( const char * fname , fnamelen ,
                       const char * vname , vnamelen ,
                       const char * aname , anamelen ,
                       char       * aval  , avallen ) ;


int wratt3c( const char          * fname ,
             const char          * vname ,
             const char          * aname ,
             int                   atyp ,
             int                   amax ,
             void                * att )

    {       /*  begin body of  wratt3c() */
    if ( atyp == M3CHAR )
        {
        return( WRATTC( fname , (int) strlen( fname ) ,
                        vname , (int) strlen( vname ) ,
                        aname , (int) strlen( aname ) ,
                        att   , amax ) ;
        } ;
    else{
        return( WRATT3( fname , (int) strlen( fname ) ,
                        vname , (int) strlen( vname ) ,
                        aname , (int) strlen( aname ) ,
                      & atyp ,
                      & amax ,
                        asiz ,
                        att ) ;
        }
    }       /*  end body of  wratt3c()  */


                	/** END  CASE OF Win32 **/
                	/** NEXT CASE:  CRAY-TARGETED wratt3c(): **/

#elif  defined(_CRAY)   /** treatment:  CRAY arryas of strings?? **/


#include <fortran.h>

    extern int WRATT3( const _fcd   fname ,
                       const _fcd   vname ,
                       const _fcd   aname ,
                       const int  * atype,
                       const int  * amax,
                       void       * aval ) ;

    extern int WRATTC( const _fcd   fname ,
                       const _fcd   vname ,
                       const _fcd   aname ,
                       _fcd         aval ) ;

int wratt3c( const char          * fname ,
             const char          * vname ,
             const char          * aname ,
             int                   atyp ,
             int                   amax ,
             void                * att )

    {       /*  begin body of  wratt3c() */

    _fcd  file ;
    _fcd  vble ;
    _fcd  atnm ;
    _fcd  aval ;

    file = _cptofcd( (char *)fname, (int) strlen( fname ) ) ;
    vble = _cptofcd( (char *)vname, (int) strlen( vname ) ) ;
    atnm = _cptofcd( (char *)aname, (int) strlen( aname ) ) ;

    if ( atyp == M3CHAR )
        {
        aval = _cptofcd( (char *)att, amax ) ;
        return( _btol( WRATT3( fname , (int) strlen( fname ) ,
                               vname , (int) strlen( vname ) ,
                               aname , (int) strlen( aname ) ,
                             & atyp ,
                             & amax ,
                               att ) ) ;
        } ;
    else{
        return( _btol( WRATTC( fname , (int) strlen( fname ) ,
                               vname , (int) strlen( vname ) ,
                               aname , (int) strlen( aname ) ,
                               att   , amax ) ) ;
        }

    }       /*  end body of  wratt3c()  */

#else

#error   "Error compiling wratt3c():  unsupported architecture"

#endif              /** IF FELDMAN-DESCENDED "FC" TARGETED, 
                        OR IF Win32, OR IF CRAY **/



