
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/wrvars.F,v 1.5 2000/12/13 21:04:32 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION WRVARS( FID, VID, FLAGS, STEP2, 
     &                           DIMS, DELS, DELTA, BUFFER )

C***********************************************************************
C  function body starts at line  86
C
C  FUNCTION:  writes "variables" part of time step records from Models-3
C             files with index FID, for all layers and variables for
C             routines WRCUSTOM, WRGWRDED, WRBNDARY, WRIDDATA, WRPROFIL,
C             and WRGRNEST.
C
C  MACHINE DEPENDENCY:
C       TYPSIZE( 6 ) must be sizeof( double )/sizeof( real )
C
C  RETURN VALUE:  TRUE iff the operation succeeds
C
C  PRECONDITIONS REQUIRED:  Should only be called by the above routines,
C             after OPEN3() has checked for file and time step availability,
C             and after the above routines have set up DIMS, , and DELTA.
C
C  SUBROUTINES AND FUNCTIONS CALLED:  (netCDF)
C
C  REVISION  HISTORY:  
C       prototype 3/92 by CJC
C
C       modified 10/94 by CJC:  writes STEP number instead of OKFLAG3;
C       supports WRITE3() operations with single-variable granularity
C
C       Modified 2/97  by CJC for OpenMP:  make it thread-safe to
C       call WRITE3 on distinct files in task-parallel.
C
C   9/99    Modified by CJC for new portability conventions
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        INTEGER       FID             !  file index within the STATE3 commons
        INTEGER       VID             !  vble index within the STATE3 commons
        INTEGER       FLAGS( 2 )      !  ( jdate yyyyddd, jtime hhmmss )
        INTEGER       STEP2           !  physical time step number (maybe mod 2)
        INTEGER       DIMS( 5 )       !  corner arg array for NCVGT()
        INTEGER       DELS( 5 )       !  diag   arg array for NCVGT()
        INTEGER       DELTA           !  d(INDX) / d(NCVGTcall)
        REAL          BUFFER(*)       !  buffer array for input


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER       IERR            !  netCDF error status return
        INTEGER       VAR             !  loop counter for file variables
        INTEGER       INDX            !  subscript location in BUFFER(*)
        INTEGER       DIMT( 5 )       !  corner:    FLAGS
        INTEGER       DELT( 5 )       !  diagonal:  FLAGS

        INTEGER         TYPSIZE( 6 )    !  sizeof( variable ) / sizeof( real )
#if _CRAY || REAL8
        DATA            TYPSIZE / 1, 1, 1, 1, 1, 1 /
#endif
#if ! ( _CRAY || REAL8 )
        DATA            TYPSIZE / 1, 1, 1, 1, 1, 2 /
#endif


C***********************************************************************
C   begin body of function  WRVARS

C...........   set up hyperslab for TFLAG

        DIMT( 1 ) = 1           !  field:  date or time
        DELT( 1 ) = 2           !  extent:  both date and time
        DIMT( 3 ) = STEP2       !  time start
        DELT( 3 ) = 1           !  time extent:  1

        DELT( 2 ) = 1           !  extent:  1


        IF ( VID .GT. 0 ) THEN          !  write just one variable

C...............   Write contents of the buffer for this variable

!$OMP CRITICAL( S_NC )
            CALL NCVPT( CDFID3( FID ), VINDX3( VID,FID ),
     &                  DIMS, DELS, BUFFER, IERR )
!$OMP END CRITICAL( S_NC )

            IF ( IERR .NE. 0 ) THEN
!$OMP CRITICAL( S_LOGOUT )
                WRITE( LOGDEV,91010 )
     &                  'Error writing variable '
     &                  // VLIST3( VID,FID ) //
     &                  'to file ' // FLIST3( FID ) ,
     &                  'netCDF error number', IERR
                WRITE( LOGDEV,* ) 'IOAPI file ID ', FID
                WRITE( LOGDEV,* ) 'netCDF ID     ', CDFID3( FID )
                WRITE( LOGDEV,* ) 'variable      ', VINDX3( VID,FID )
                WRITE( LOGDEV,* ) 'dims array    ', DIMS
                WRITE( LOGDEV,* ) 'delts array   ', DELS
                WRITE( LOGDEV,* )
!$OMP END CRITICAL( S_LOGOUT )
        
                WRVARS = .FALSE.
                RETURN
        
            END IF          !  ierr nonzero:  NCVPT() failed

C...............   Write time step flag:

            DIMT( 2 ) = VID    !  starting variable
            
!$OMP CRITICAL( S_NC )
            CALL NCVPT( CDFID3( FID ), TINDX3( FID ),
     &                  DIMT, DELT, FLAGS, IERR )
!$OMP END CRITICAL( S_NC )
        
            IF ( IERR .NE. 0 ) THEN
!$OMP CRITICAL( S_LOGOUT )
                WRITE( LOGDEV,91010 )
     &              'Error writing timestep flag for vble '
     &              // VLIST3( VID,FID ) //
     &              'from file ' // FLIST3( FID ) ,
     &              'netCDF error number', IERR
                WRITE( LOGDEV,* ) 'IOAPI file ID ', FID
                WRITE( LOGDEV,* ) 'netCDF ID     ', CDFID3( FID )
                WRITE( LOGDEV,* ) 'variable      ', VINDX3( VID,FID )
                WRITE( LOGDEV,* ) 'dims array    ', DIMT
                WRITE( LOGDEV,* ) 'delts array   ', DELT
                WRITE( LOGDEV,* )
!$OMP END CRITICAL( S_LOGOUT )
        
                WRVARS = .FALSE.
                RETURN
        
            END IF                      !  if ierr nonzero:  NCVPT() failed

        ELSE                            !  write all variables:
        
C...............   Write contents of the buffer for each variable

            INDX = 1                    !  starting subscript for BUFFER(*)
            
            DO  12  VAR = 1 , NVARS3( FID )
                   
!$OMP CRITICAL( S_NC )
                CALL NCVPT( CDFID3( FID ), VINDX3( VAR,FID ),
     &                      DIMS, DELS, BUFFER( INDX ), IERR )
!$OMP END CRITICAL( S_NC )
                IF ( IERR .NE. 0 ) THEN
!$OMP CRITICAL( S_LOGOUT )
                    WRITE( LOGDEV,91010 )
     &                  'Error writing variable '
     &                  // VLIST3( VAR,FID ) //
     &                  'to file ' // FLIST3( FID ) ,
     &                  'netCDF error number', IERR
                    WRITE( LOGDEV,* ) 'IOAPI ID    ', FID
                    WRITE( LOGDEV,* ) 'netCDF ID   ', CDFID3( FID )
                    WRITE( LOGDEV,* ) 'vble        ', VINDX3( VAR,FID )
                    WRITE( LOGDEV,* ) 'dims array  ', DIMS
                    WRITE( LOGDEV,* ) 'delts array ', DELS
                    WRITE( LOGDEV,* ) 'offset      ', INDX
                    WRITE( LOGDEV,* ) 'WRITE3 request: ALL VARIABLES'
                    WRITE( LOGDEV,* )
!$OMP END CRITICAL( S_LOGOUT )
        
                    WRVARS = .FALSE.
                    RETURN
        
                END IF                  !  ierr nonzero:  NCVPT() failed
                   
                INDX = INDX  +  DELTA * TYPSIZE( VTYPE3( VAR,FID ) )
                   
C...............   Write time step flags:
      
                DIMT( 2 ) = VAR         !  variable-number

!$OMP CRITICAL( S_NC  )
                CALL NCVPT( CDFID3( FID ), TINDX3( FID ),
     &                      DIMT, DELT, FLAGS, IERR )
!$OMP END CRITICAL( S_NC )
                IF ( IERR .NE. 0 ) THEN
!$OMP CRITICAL( S_LOGOUT )
                    WRITE( LOGDEV,91010 )
     &                  'Error writing timestep flags for file ' // 
     &                  FLIST3( FID ) ,
     &                  'netCDF error number', IERR
                    WRITE( LOGDEV,* ) 'IOAPI file ID ', FID
                    WRITE( LOGDEV,* ) 'netCDF ID     ', CDFID3( FID )
                    WRITE( LOGDEV,* ) 'dims array    ', DIMT
                    WRITE( LOGDEV,* ) 'delts array   ', DELT
                    WRITE( LOGDEV,* ) 'WRITE3 request: ALL VARIABLES'
                    WRITE( LOGDEV,* )
!$OMP END CRITICAL( S_LOGOUT )

                    WRVARS = .FALSE.
                    RETURN

                END IF                  !  ierr nonzero:  NCVPT() failed

12          CONTINUE                    !  end loop on variables VAR

            IF ( NVARS3( FID ) .EQ. 0 ) THEN	!  write all-vars timestep flag

                DIMT( 2 ) = 1           !  variable-number

!$OMP CRITICAL( S_NC )
                CALL NCVPT( CDFID3( FID ), TINDX3( FID ),
     &                      DIMT, DELT, FLAGS, IERR )
!$OMP END CRITICAL( S_NC )
                IF ( IERR .NE. 0 ) THEN
!$OMP CRITICAL( S_LOGOUT )
                    WRITE( LOGDEV,91010 )
     &                  'Error writing timestep flags for file ' // 
     &                  FLIST3( FID ) ,
     &                  'netCDF error number', IERR
                    WRITE( LOGDEV,* ) 'IOAPI file ID ', FID
                    WRITE( LOGDEV,* ) 'netCDF ID     ', CDFID3( FID )
                    WRITE( LOGDEV,* ) 'dims array    ', DIMT
                    WRITE( LOGDEV,* ) 'delts array   ', DELT
                    WRITE( LOGDEV,* ) 'WRITE3 request: ALL VARIABLES'
                    WRITE( LOGDEV,* )
!$OMP END CRITICAL( S_LOGOUT )

                    WRVARS = .FALSE.
                    RETURN

                END IF                  !  ierr nonzero:  NCVPT() failed

            END IF		!  if nvars3 zero
        
        END IF                  !  if writing just one vble, or all vbles
        
        IF ( VOLAT3( FID ) ) THEN     !  volatile file:  synch with disk
            
!$OMP CRITICAL( S_NC )
            CALL NCSNC( CDFID3( FID ), IERR )
!$OMP END CRITICAL( S_NC )
            IF ( IERR .NE. 0 ) THEN
!$OMP CRITICAL( S_LOGOUT )
                WRITE( LOGDEV,91030 )
     &              'netCDF error number', IERR,
     &              'Error with disk synchronization for file:  '
     &              // FLIST3( FID )
     &          
!$OMP END CRITICAL( S_LOGOUT )
 
               WRVARS = .FALSE.
               RETURN
 
            END IF      !  if NCSNC() error
 
        END IF          !  if file volatile

        WRVARS = .TRUE.         ! (if you get to here)
        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx
 
91010   FORMAT ( //5X , '>>> WARNING in subroutine WRVARS <<<',
     &            2 ( /5X , A , : ) , I5, // )
 
91030   FORMAT ( //5X , '>>> WARNING in subroutine WRITE3 <<<',
     &            /5X , A , I5 , : ,
     &            /5X , A ) 

        END

