
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/xtbuf3.f,v 1.2 2000/11/28 21:23:10 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION  XTBUF3( FID, VID, 
     &                            LAY0, LAY1, ROW0, ROW1, COL0, COL1,
     &                            JDATE, JTIME,
     &                            BUFFER )

C***********************************************************************
C  function body starts at line  77
C
C  FUNCTION:  reads data from Models-3 BUFFEREd "file" with M3 file
C       index FID for variable with name VNAME and indicated col-row-layer
C       window for the date and time JDATE (coded YYYYDDD) JTIME (HHMMSS).
C       For time-independent files, JDATE:JTIME are ignored.
C       If VNAME is 'ALL', reads all variables; if LAYER is -1,
C       reads all layers.
C
C  RETURN VALUE:  TRUE iff the operation succeeds (and the data is available)
C
C  PRECONDITIONS REQUIRED:  FNAME is a Models-3 BUFFERED "file" already 
C       opened by CREATE3().  Should only be called via XTRACT3().
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       JSTEP3, BUFGET3
C
C  REVISION  HISTORY:  
C       Prototype 7/94 by CJC
C	Modified 10/94 by CJC to work with WRITE3() having granularity
C	at the level of individual variables.
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'

C...........   ARGUMENTS and their descriptions:

        INTEGER       FID             !  file     id for STATE3 arrays
        INTEGER       VID             !  variable id  "     "     "
        INTEGER       LAY0            !  lower layer  bound
        INTEGER       LAY1            !  upper layer  bound
        INTEGER       ROW0            !  lower row    bound
        INTEGER       ROW1            !  upper row    bound
        INTEGER       COL0            !  lower column bound
        INTEGER       COL1            !  upper column bound
        INTEGER       JDATE           !  date, formatted YYYYDDD
        INTEGER       JTIME           !  time, formatted HHMMSS
        REAL          BUFFER(*)       !  input buffer array


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER       JSTEP3     !  compute time step record numbers
        INTEGER       BUFXTR3    !  from bufint3.c

        EXTERNAL      JSTEP3, BUFXTR3


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER       STEP            !  time step record number
        INTEGER       VAR             !  loop counter (over variables)


C***********************************************************************
C   begin body of function  XTBUF3

        IF ( VID .GT. 0 ) THEN		!  xtract on just this variable
        
            IF ( TSTEP3( FID ) .EQ. 0 ) THEN
                   
                IF( LDATE3( VID,FID ) .EQ. 0 ) THEN
                    STEP = ILAST3( VID,FID )
                ELSE
                    XTBUF3 = .FALSE.
                    RETURN
                END IF
                
            ELSE IF ( JDATE .EQ. LDATE3( VID,FID ) .AND. 
     &                JTIME .EQ. LTIME3( VID,FID ) ) THEN
                
                STEP = ILAST3( VID,FID )
                
            ELSE IF ( JDATE .EQ. NDATE3( VID,FID ) .AND. 
     &                JTIME .EQ. NTIME3( VID,FID ) ) THEN
                
                STEP = 1 - ILAST3( VID,FID )        !  formula swaps 0 and 1
                
            ELSE
                
                CALL M3EXIT( 
     &                  'XTRACT3/XTBUF3', JDATE, JTIME,
     &                  'Date and time not available for '// 
     &                  VLIST3( VID,FID ) // ' in ' // FLIST3( FID ),
     &                  2 )
                
                XTBUF3 = .FALSE.
                RETURN
                
            END IF
                
            XTBUF3 = ( 0 .NE. BUFXTR3( FID, VID, 
     &               LAY0, LAY1, ROW0, ROW1, COL0, COL1,
     &               NLAYS3( FID ), NROWS3( FID ), NCOLS3( FID ),
     &               STEP, BUFFER ) )
                
        ELSE				!  xtract on all variables
        
            DO  11  VAR = 1, NVARS3( FID )
        
                IF ( TSTEP3( FID ) .EQ. 0 ) THEN
                       
                    IF( LDATE3( VAR,FID ) .EQ. 0 ) THEN
                        STEP = ILAST3( VAR,FID )
                    ELSE
                        XTBUF3 = .FALSE.
                        RETURN
                    END IF
                    
                ELSE IF ( JDATE .EQ. LDATE3( VAR,FID ) .AND. 
     &                    JTIME .EQ. LTIME3( VAR,FID ) ) THEN
                    
                    STEP = ILAST3( VAR,FID )
                    
                ELSE IF ( JDATE .EQ. NDATE3( VAR,FID ) .AND. 
     &                    JTIME .EQ. NTIME3( VAR,FID ) ) THEN
                    
                    STEP = 1 - ILAST3( VAR,FID )        !  formula swaps 0 and 1
                    
                ELSE
                    
                    CALL M3WARN( 'XTRACT3/XTBUF3', JDATE, JTIME,
     &                   'Date and time not available for ' //
     &                   VLIST3( VAR,FID ) // ' in ' // FLIST3( FID ) )
                    
                    XTBUF3 = .FALSE.
                    RETURN
                    
                END IF
                    
                IF ( 0 .EQ. BUFXTR3( FID, VAR, LAY0, LAY1, ROW0, ROW1, 
     &                               COL0, COL1, NLAYS3( FID ), 
     &                               NROWS3( FID ), NCOLS3( FID ), 
     &                               STEP, BUFFER ) ) THEN
                    
                    XTBUF3 = .FALSE.
                    RETURN
            
                END IF		!  if bufxtr3() failed.
            
11          CONTINUE
                
        END IF		!  if vid > 0, or not
        
        RETURN
        END

