
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi/./ioapi/src/xtract3.f,v 1.2 2000/11/28 21:23:10 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION  XTRACT3 ( FNAME, VNAME, LAY0, LAY1,
     &                              ROW0, ROW1, COL0, COL1,
     &                              JDATE, JTIME, BUFFER )

C***********************************************************************
C  function body starts at line  111
C
C  FUNCTION:  Read into the array BUFFER(*) all the data from the
C             Models-3 data file with logical name FNAME for variable
C             with name VNAME, for timestep JDATE:JTIME (formatted
C             YYYYDDD,HHMMSS), that fits in the data window :
C
C                  LAY0 <= LAY <= LAY1
C                  ROW0 <= ROW <= ROW1
C                  COL0 <= COL <= COL1
C
C
C  RETURN VALUE:  TRUE iff the data is available and the operation succeeds.
C
C  PRECONDITIONS REQUIRED:  FNAME is the logical name of a Models-3 data
C              file already opened by OPEN3() or CREATE3().
C
C  SUBROUTINES AND FUNCTIONS CALLED:  INDEX1, INIT3, JSTEP3, RDVARS
C
C  REVISION  HISTORY:
C       prototype 3/92 by CJC
C
C       modified  7/94 by CJC to handle restart files (tstep < 0 )
C
C       modified  8/94 by CJC to handle BUFFERED "files"
C
C       modified 10/94 by CJC to handle WRITE3() granularity at
C       the level of individual variables; char*(*) name arguments.
C
C       Modified  5/98, 1/00  by CJC for OpenMP thread-safety
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*) FNAME           !  logical file name
        CHARACTER*(*) VNAME           !  variable name, or 'ALL'
        INTEGER       LAY0            !  lower layer bound
        INTEGER       LAY1            !  upper layer bound
        INTEGER       ROW0            !  lower row bound
        INTEGER       ROW1            !  upper row bound
        INTEGER       COL0            !  lower col bound
        INTEGER       COL1            !  upper col bound
        INTEGER       JDATE           !  date, formatted YYYYDDD
        INTEGER       JTIME           !  time, formatted HHMMSS
        REAL          BUFFER(*)       !  input buffer array


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER       INDEX1     !  look up names in name tables
        INTEGER       INIT3      !  initialize I/O system files.
        INTEGER       JSTEP3     !  compute time step record numbers
        INTEGER       XTBUF3     !  read data window from BUFFERED "files"
        LOGICAL       RDVARS     !  read variables in data window

        EXTERNAL      INDEX1, INIT3, JSTEP3, RDVARS, XTBUF3
        EXTERNAL      INITBLK3   !  block data: initialize I/O state


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER       FLAG1, FLAG2       !  current time step flag
        INTEGER       FLAGS( 2,MXVARS3 ) !  array of time step flags
        INTEGER       FID             !  file-subscript for STATE3 arrays
        INTEGER       VID             !  vble-subscript for STATE3 arrays
        INTEGER       VAR             !  loop counter (subscript)
        INTEGER       STEP            !  time step record number
        INTEGER       STEP2           !  1 or 2, according to step mod 2
        INTEGER       IERR            !  netCDF error status return
        INTEGER       DELTA           !  volume of a NCVGT call
        INTEGER       DCOLS           !  column-size of read-window
        INTEGER       DROWS           !  row-size of read-window
        INTEGER       DLAYS           !  layer-size of read-window
        INTEGER       DIMS( 5 )       !  corner arg array for NCVGT()
        INTEGER       DELS( 5 )       !  corner arg array for NCVGT()
        INTEGER       DIMT( 5 )       !  corner arg array for NCVGT()
        INTEGER       DELT( 5 )       !  corner arg array for NCVGT()
        CHARACTER*16  FIL16           !  scratch file-name buffers
        CHARACTER*16  VAR16           !  scratch vble-name buffers
        CHARACTER*256 MESG            !  message/warning buffer
        LOGICAL       EFLAG


C***********************************************************************
C   begin body of function  XTRACT3

C.......   Check that Models-3 I/O API has been initialized:

        EFLAG = .FALSE.
!$OMP   CRITICAL( S_LOGDEV )
        IF ( .NOT. FINIT3 ) THEN
            LOGDEV  = INIT3()
            XTRACT3 = .FALSE.
            EFLAG   = .TRUE.
        END IF          !  if not FINIT3
!$OMP   END CRITICAL( S_LOGDEV )
        IF ( EFLAG ) THEN
            CALL M3MSG2(  'XTRACT3:  I/O API not yet initialized.' )
            RETURN
        END IF
        

C.......   Check lengths of string arguments:

        IF ( LEN( FNAME ) .GT. 16 ) THEN
            WRITE( MESG, '( 6A, I5 )' )
     &          'File "', FNAME, '", Variable "', VNAME, '"',
     &          'Max file name length 16; actual:', LEN( FNAME )
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN
        ELSE IF ( LEN( VNAME ) .GT. 16 ) THEN
            WRITE( MESG, '( 6A, I5 )' )
     &          'File "', FNAME, '", Variable "', VNAME, '"',
     &          'Max vble name length 16; actual:', LEN( VNAME )
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN
        END IF          !  if len( fname ) > 16, or if len( vname ) > 16
        

C.......   Find netCDF index for the file, and check time step availability:

        FIL16 = FNAME   !  fixed-length-16 scratch copy of name
        FID   = INDEX1( FIL16, COUNT3, FLIST3 )
        IF ( FID .EQ. 0 ) THEN  !  file not available

            MESG =  'File:  '//FIL16// ' not yet opened.'
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN

        ELSE IF ( FTYPE3( FID ) .NE. GRDDED3 ) THEN

            WRITE( MESG, '( 4A, I3)' )
     &          'File type error in XTRACT3:  ',
     &          FIL16, ' must be GRDDED3 = 1 ',
     &          'Actual file type' , FTYPE3( FID )
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN

        ELSE IF ( VOLAT3( FID ) ) THEN     !  volatile file:  synch with disk

!$OMP CRITICAL( S_NC )
            CALL NCSNC( CDFID3( FID ), IERR )
            IF ( IERR .NE. 0 ) THEN

                WRITE( MESG, '( 3A, I7 )' )
     &          'Error with disk synchronization for file:  ',
     &          FIL16 , 'netCDF error number', IERR
                CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
                XTRACT3 = .FALSE.

            END IF      !  if ncsnc() failed
!$OMP END CRITICAL( S_NC )
            IF ( IERR .NE. 0 ) RETURN

        END IF          !  if file type nongridded, or file volatile


C.......   Perform range checks on variable, layer, row, col:

        VAR16 = VNAME   !  fixed-length-16 scratch copy of name
        IF ( VAR16 .EQ. ALLVAR3 ) THEN

            VID = ALLAYS3

        ELSE 
            
            VID = INDEX1 ( VAR16, NVARS3( FID ), VLIST3( 1,FID ) )
            
            IF ( VID .EQ. 0 ) THEN
                MESG = 'Requested variable ' //  VAR16 //
     &                 'not available in file ' // FLIST3( FID ) //
     &                 'which contains variables:'
                CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
                DO  VAR = 1, NVARS3( FID )
                     CALL M3MSG2( VLIST3( VAR,FID ) )
                END DO
                XTRACT3 = .FALSE.
                RETURN
            END IF
 
        END IF          !  end check on variable
 
        IF ( LAY0 .LT. 1     .OR.  
     &       LAY1 .LT. LAY0  .OR. 
     &       LAY1 .GT. NLAYS3( FID ) ) THEN
        
            WRITE( MESG, '( A, I9 )' ) 
     &          'Lower bound:            ', LAY0
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I9 )' ) 
     &          'Upper bound:            ', LAY1
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I9 )' ) 
     &          'Actual number of layers:', NLAYS3( FID )
            CALL M3MSG2( MESG )
            MESG = 'Error in layer-bounds specification for file ' //
     &          FIL16 // ' variable ' // VAR16
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN
     
        ELSE IF ( ROW0 .LT. 1     .OR.  
     &            ROW1 .LT. ROW0  .OR. 
     &            ROW1 .GT. NROWS3( FID ) ) THEN
        
            WRITE( MESG, '( A, I9 )' ) 
     &          'Lower bound:            ', ROW0
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I9 )' ) 
     &          'Upper bound:            ', ROW1
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I9 )' ) 
     &          'Actual number of layers:', NROWS3( FID )
            CALL M3MSG2( MESG )
            MESG = 'Error in row-bounds specification for file ' //
     &          FIL16 // ' variable ' // VAR16
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN
     
        ELSE IF ( COL0 .LT. 1     .OR.  
     &            COL1 .LT. COL0  .OR. 
     &            COL1 .GT. NCOLS3( FID ) ) THEN
        
            WRITE( MESG, '( A, I9 )' ) 
     &          'Lower bound:            ', COL0
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I9 )' ) 
     &          'Upper bound:            ', COL1
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I9 )' ) 
     &          'Actual number of layers:', NCOLS3( FID )
            CALL M3MSG2( MESG )
            MESG = 'Error in col-bounds specification for file ' //
     &          FIL16 // ' variable ' // VAR16
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN

        END IF          !  end range checks on col, row, layer
 
     
        IF ( CDFID3( FID ) .EQ. BUFFIL3 ) THEN     !  BUFFERED "file"

            XTRACT3 = ( 0 .NE. XTBUF3( FID,  VID,  LAY0, LAY1, 
     &                                 ROW0, ROW1, COL0, COL1,
     &                                 JDATE, JTIME, BUFFER ) )
            RETURN
        
        END IF !  if file buffered


        STEP = JSTEP3( JDATE, JTIME,
     &                 SDATE3( FID ),
     &                 STIME3( FID ),
     &                 TSTEP3( FID ) )

        IF ( STEP .LT. 0 ) THEN

            WRITE( MESG, '( A, I9, A, I6.6 )' )
     &          'File starting date & time:',
     &          SDATE3( FID ), ':', STIME3( FID )
            CALL M3MSG2( MESG )
            WRITE( MESG, '( A, I6.6 )' )
     &          'File time step:           ', TSTEP3( FID )
            MESG = 'Time step error reading file:  ' // FIL16 
            CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
            XTRACT3 = .FALSE.
            RETURN

        END IF          !  if step , 0


        IF ( TSTEP3( FID ) .LT. 0 ) THEN
            STEP2 = 1 + MOD( STEP - 1, 2 )
            FLAG1 = JDATE
            FLAG2 = JTIME
            
        ELSE IF ( TSTEP3( FID ) .GT. 0 ) THEN
            STEP2 = STEP
            FLAG1 = JDATE
            FLAG2 = JTIME
        ELSE    ! tstep3( fid ) = 0
            STEP2 = STEP
            FLAG1 = 0
            FLAG2 = 0
        END IF
      
        DIMT( 1 ) = 1           !  field:  date or time
        DELT( 1 ) = 2           !  extent:  both date and time
        DIMT( 3 ) = STEP2       !  initial timestep
        DELT( 3 ) = 1           !  extent:  one step
        
        IF ( VID .GT. 0 ) THEN  !  reading just one variable
               
            DIMT( 2 ) = VID     !  initial variable number
            DELT( 2 ) = 1       !  extent:  one variable
        
!$OMP CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ), TINDX3( FID ), 
     &                  DIMT, DELT, FLAGS( 1,VID ), IERR )
!$OMP END CRITICAL( S_NC )
            IF ( IERR .EQ. 8 ) THEN     !  timestep flag not yet written
               
                XTRACT3 = .FALSE.
                RETURN
                        
            ELSE IF ( IERR .NE. 0 ) THEN
               
                WRITE( MESG, '( A, I9, 4( 1X, A ) )' )
     &              'netCDF error number', IERR,
     &              'reading netCDF time step flag for file', FIL16,
     &              'variable', VAR16
                CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )       
                XTRACT3 = .FALSE.
                RETURN
               
            ELSE IF( FLAGS( 1,VID ) .NE. FLAG1 .OR.
     &               FLAGS( 2,VID ) .NE. FLAG2 ) THEN
               
                MESG = 'Time step not available for file ' // 
     &                 FIL16 // ' variable ' // VAR16
                CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )
                XTRACT3 = .FALSE.
                RETURN
               
            END IF          !  if ierr nonzero or not for NCVGT1()
                   
        ELSE            !  reading all variables
        
            DIMT( 2 ) = 1               !  initial variable
            DELT( 2 ) = MAX( 1, NVARS3( FID ) )   !  extent:  all variables
        
!$OMP CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ), TINDX3( FID ), 
     &                  DIMT, DELT, FLAGS, IERR )
!$OMP END CRITICAL( S_NC )
        
            IF ( IERR .EQ. 8 ) THEN     !  timestep flag not yet written
               
                XTRACT3 = .FALSE.
                RETURN
                        
            ELSE IF ( IERR .NE. 0 ) THEN
               
                WRITE( MESG, '( A, I9, 3( 1X, A ) )' )
     &              'netCDF error number', IERR,
     &              'reading netCDF time step flag for file', FIL16,
     &              'ALL variables'
                CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )       
                XTRACT3 = .FALSE.
                RETURN
               
            END IF          !  if ierr nonzero or not for NCVGT()

            IF ( NVARS3( FID ) .GT. 0 ) THEN

                DO  11  VAR = 1, NVARS3( FID )

                    IF ( FLAGS( 1,VAR ) .NE. FLAG1  .OR.
     &                   FLAGS( 2,VAR ) .NE. FLAG2  ) THEN

                        MESG = 'Variable ' // VLIST3( VAR, FID ) //
     &                         ' not available in file ' // FIL16
                        CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )       
                        XTRACT3 = .FALSE.
                        RETURN

                    END IF          !  if bad flag value

11              CONTINUE

            ELSE	!  nvars zero (structured no-user-vble time step)

                IF ( FLAGS( 1,1 ) .NE. FLAG1  .OR.
     &               FLAGS( 2,1 ) .NE. FLAG2  ) THEN

                    MESG = 'Reading ALL VARIABLES: ' //
     &                     'Time step not available in file ' // FIL16
                    CALL M3WARN( 'XTRACT3', JDATE, JTIME, MESG )       
                    XTRACT3 = .FALSE.
                    RETURN

                END IF          !  if bad flag value

            END IF	!  if nvars positive, or not

        END IF          !  if reading one variable or many (checking flags)


C...........   Read data from file into BUFFER()

        DCOLS = COL1  -  COL0 + 1
        DROWS = ROW1  -  ROW0 + 1
        DLAYS = LAY1  -  LAY0 + 1

        DIMS( 1 ) = COL0
        DELS( 1 ) = DCOLS

        DIMS( 2 ) = ROW0
        DELS( 2 ) = DROWS

        DIMS( 3 ) = LAY0
        DELS( 3 ) = DLAYS

        DIMS( 4 ) = STEP
        DELS( 4 ) = 1

        DELTA = DCOLS * DROWS * DLAYS

        XTRACT3 = RDVARS( FID, VID, DIMS, DELS, DELTA, BUFFER )

        RETURN

        END

