
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi_tools/./ioapi_tools/src/m3tproc.f,v 1.4 2000/12/15 22:55:13 smith_w Exp $"
C EDSS/Models-3 I/O API.  Copyright (C) 1992-2001 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................


        PROGRAM  M3TPROC

C***********************************************************************
C  program body starts at line 136
C
C  FUNCTION:
C       Sums, give max, or gives average over a specified time period 
C       for a subset of variables from the input file, and writes the 
C       processed data to the output file.
C
C  PRECONDITIONS REQUIRED:
C       Machine with stack-allocated AUTO local variables (e.g., CRAY)
C       consistency with FORIO:PARMS3.EXT for name and description lengths.
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       GETNUM, NEXTIME, Models-3 I/O.
C
C  REVISION  HISTORY:
C      Prototype 5/1997 by M Houyoux
C       Version 11/2001 by Carlie J. Coats, Jr., for I/O API Version 2.1
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'  !  I/O parameter definitions
        INCLUDE 'FDESC3.EXT'  !  file header data structures
        INCLUDE 'IODECL3.EXT' !  I/O definitions and declarations


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER       ENVINT
        LOGICAL       ENVYN
        INTEGER       GETDATE
        INTEGER       GETMENU
        INTEGER       GETNUM
        REAL          GETREAL
        LOGICAL       GETYN
        INTEGER       IARGC
        INTEGER       INDEX1
        INTEGER       JULIAN
        CHARACTER*16  PROMPTMFILE
        INTEGER       SEC2TIME
        INTEGER       SECSDIFF
        INTEGER       TIME2SEC
        INTEGER       TRIMLEN

        EXTERNAL  ENVINT, ENVYN, GETDATE, GETMENU, GETNUM, GETREAL,
     &            GETYN, IARGC, INDEX1, JULIAN, PROMPTMFILE, 
     &            SEC2TIME, SECSDIFF, TIME2SEC, TRIMLEN

C...........   PARAMETERS and their descriptions:
 
        CHARACTER*16    BLANK16
        INTEGER         M3AVE
        INTEGER         M3MAX
        INTEGER         M3SUM
        INTEGER         MAXITEMS

        PARAMETER     ( BLANK16 = ' ',
     &                  M3SUM = 1,
     &                  M3AVE = 2,
     &                  M3MAX = 3,
     &                  MAXITEMS = 3  )
     
        CHARACTER*80    PROGVER
        DATA PROGVER /
     &  '$Id$'
     &  /

C...........   LOCAL VARIABLES and their descriptions:

        CHARACTER*16    ANAME   !  scratch buffer for variable names
        CHARACTER*16    INAME   !  logical name of the  input file
        CHARACTER*16    ONAME   !  logical name of the output file
        CHARACTER*16    UNITS ( MXVARS3 ) !  list of vble units
        CHARACTER*16    VNAMEI( MXVARS3 ) !  list of vble names, from user
        CHARACTER*16    VNAMEO( MXVARS3 ) !  list of vble names, from user
        CHARACTER*80    ALINE   !  scratch buffer for prompt
        CHARACTER*80    VDESC ( MXVARS3 ) !  list of vble descs
        CHARACTER*256   ENVBUF  !  value from command line arguments
        CHARACTER*256   MESG    !  for M3WARN(), M3EXIT()

        INTEGER         I, T, V !  loop counter (time step #)

        INTEGER         ARGCNT  !  number of command-line args, from IARGC()
        INTEGER         BDATE   !  beginning analysis window date
        INTEGER         BTIME   !  beginning analysis window time
        INTEGER         DAY     !  temporary day
        INTEGER         DMAX    !  string length for descriptions
        INTEGER         IOS     !  I/O status
        INTEGER         JDATE   !  current  input date
        INTEGER         JTIME   !  current  input time
        INTEGER         LAYER   !  layer number for processing (or all)
        INTEGER         LOGDEV  !  unit number for log file
        INTEGER         MON     !  temporary month 
        INTEGER         NVSAV   !  number of variables in input file
        INTEGER         NVARS   !  number of vbles in ONAME 
        INTEGER         PERIOD  !  Period length (HHMMSS) for repetitive analysis 
        INTEGER         PERSEC  !  Period length in seconds
        INTEGER         PDATE   !  current output date
        INTEGER         PTIME   !  current output time
        INTEGER         SIZE    ! volume of one variable
        INTEGER         SDATE   !  starting  input date, from user
        INTEGER         STIME   !  starting  input time, from user
        INTEGER         RUNLEN  !  duration, HHMMSS from user
        INTEGER         TSTEP   !  time step, from INAME header
        INTEGER         ATYPE   !  type of aggregation being performed
        INTEGER         UMAX    !  string length for units
        INTEGER         WSTEPS  !  window duration in TSTEPs
        INTEGER         VMAX    !  string length for names
        INTEGER         WINLEN  !  duration (HHMMSS) of analysis window
        INTEGER         YR      !  temporary year

        LOGICAL         NPFLAG  !  iff no prompting for variables

        CHARACTER*80    MENUITMS( MAXITEMS )!  buffer for operations menu items 
        DATA            MENUITMS /
     &           'Sum over time window',
     &           'Calculate average over time window',
     &           'Determine maximum over time window'   /

C.........................................................................
C   begin body of program  M3TPROC

        LOGDEV = INIT3()
        WRITE ( *,92000 )
     &  ' ',
     &  'Program M3TPROC to sum, average, or find the maximum values',
     &  'over a repeating time period from a selected time window.',
     &  'The time period and starting time window set the start and',
     &  'duration of all subsequent time windows. The program inputs',
     &  'and outputs Models-3 files.',
     &  ' ',
     &  'You need to have set environment variables for the input',
     &  'and output file logical names.  You will be asked to select',
     &  'the time period to be copied and the start of the time ',
     &  'period to receive the results.',
     &  ' ',
     &  'USAGE:  m3tproc [INFILE OUTFILE] ',
     &  '(and then answer the prompts).',
     &  ' ',
     &'See URL  http://www.emc.mcnc.org/products/ioapi/AA.html#tools',
     &' ',
     &'Program copyright (C) 2001 MCNC and released under Version 2',
     &'of the GNU General Public License.  See enclosed GPL.txt, or',
     &'URL  http://www.gnu.org/copyleft/gpl.html',
     &'Comments and questions are welcome and can be sent to',
     &' ',
     &'    envpro@emc.mcnc.org',
     &' ',
     &'    MCNC -- Environmental Modeling Center',
     &'    3021 Cornwallis Rd    P. O. Box 12889',
     &'    Research Triangle Park, NC 27709-2889',
     &' ',
     &'Program version: ' // PROGVER, 
     &' ',
     &'Program release tag: $Name$', 
     &' '

        ARGCNT = IARGC()
 
        IF ( ARGCNT .EQ. 1  .OR.  ARGCNT .GT. 2 ) THEN
            CALL M3EXIT( 'M3TPROC', 0, 0, 
     &                   'usage:  m3tproc [INFILE OUTFILE]', 2 )
        END IF
 
        IF ( ARGCNT .EQ. 0 ) THEN       !  get names from user
 
            INAME = PROMPTMFILE( 'Enter logical name for  INPUT FILE',
     &                           FSREAD3, 'INFILE', 'M3TPROC' )

        ELSE		!  argcnt 2

            CALL GETARG( 1, ENVBUF )
            INAME = ENVBUF( 1:16 )
            IF ( .NOT. OPEN3( INAME, FSREAD3, 'M3TPROC' ) ) THEN
                MESG = 'Could not open input file "' 
     &                       // INAME( 1:TRIMLEN( INAME ) ) // '"'
                CALL M3EXIT( 'M3TPROC', 0, 0, MESG, 2 )
            END IF

            CALL GETARG( 2, ENVBUF )
            ONAME = ENVBUF( 1:16 )

        END IF


        IF ( .NOT. DESC3( INAME ) ) THEN
            MESG = 'Could not get description of input file "' //
     &             INAME( 1:TRIMLEN( INAME ) ) // '"'
            CALL M3EXIT( 'M3TPROC', 0, 0, MESG, 2 )
        END IF

        LAYER = ALLAYS3
 
        IF ( FTYPE3D .EQ. CUSTOM3 ) THEN
            SIZE = NCOLS3D * NLAYS3D * NVARS3D 
        ELSE IF ( FTYPE3D .EQ. GRDDED3 ) THEN
            SIZE = NCOLS3D * NROWS3D * NLAYS3D * NVARS3D
        ELSE IF ( FTYPE3D .EQ. BNDARY3 ) THEN
            SIZE = 2 * NTHIK3D * NLAYS3D * NVARS3D 
     &               * ( NCOLS3D + NROWS3D + 2 )
        ELSE IF ( FTYPE3D .EQ. IDDATA3 ) THEN
            SIZE = 1 + NROWS3D * NVARS3D * ( NLAYS3D + 1 )
        ELSE IF ( FTYPE3D .EQ. PROFIL3 ) THEN
            SIZE = 1 + NROWS3D * ( 7 + NVARS3D * NLAYS3D )
        ELSE IF ( FTYPE3D .EQ. GRNEST3 ) THEN
            SIZE = 1 + NROWS3D * ( 12 + NCOLS3D * NVARS3D )
        ELSE
            WRITE( MESG, 94011 )
     &      'Input file "', INAME( 1:TRIMLEN( INAME ) ), 
     &      '" has unsupported type', FTYPE3D
            CALL M3EXIT( 'M3TPROC', 0, 0, MESG, 2 )
        END IF

        NVSAV  = NVARS3D
        SDATE  = SDATE3D
        STIME  = STIME3D
        TSTEP  = TSTEP3D

C.......   Get max string-lengths for use in variables-listing:
 
        VMAX = TRIMLEN( VNAME3D( 1 ) )
        UMAX = TRIMLEN( UNITS3D( 1 ) )
        DMAX = TRIMLEN( VDESC3D( 1 ) )
        DO  11  V = 1, NVARS3D
            VMAX = MAX( VMAX , TRIMLEN( VNAME3D( V ) ) )
            UMAX = MAX( UMAX , TRIMLEN( UNITS3D( V ) ) )
            DMAX = MAX( DMAX , TRIMLEN( VDESC3D( V ) ) )
11      CONTINUE

C.......  Determine if all variables are to be used

        NPFLAG = ENVYN( 'M3TPROC_ALLV', 
     &                  'true if no prompting for variables to output',
     &                  .FALSE., IOS )

C.......  If no prompting set to total number of vars, or prompt
        IF( NPFLAG ) THEN

            NVARS = NVARS3D

            DO 22 V = 1, NVARS3D
               VNAMEI( V )  = VNAME3D( V )
               VNAMEO( V )  = VNAME3D( V )
               UNITS ( V )  = UNITS3D( V )
               VDESC ( V )  = VDESC3D( V )
22          CONTINUE

        ELSE
 
            NVARS = 0
            V     = 0
 
111         CONTINUE        !  loop getting variables-list for extraction
 
                IF( MOD( NVARS,10 ) .EQ. 0 ) THEN
                    WRITE( *,92000 )
     &              ' ', 'The list of variables in file "'
     &              // INAME( 1:TRIMLEN( INAME ) ) // '" is:', ' '
                    WRITE( *,92010 )
     &              ( I,
     &                VNAME3D( I )( 1:VMAX ) // ' (' //
     &                UNITS3D( I )( 1:UMAX ) // '): ' //
     &                VDESC3D( I )( 1:DMAX ), I = 1, NVSAV  )
                END IF
 
                V = GETNUM( 0, NVSAV, 1 + MOD( V, NVSAV ),
     &            'Enter number for variable to extract (0 to quit)' )
 
                IF ( V .EQ. 0 ) GO TO  199      !  to end of loop
 
                NVARS = NVARS + 1

C...............   Optional renaming of this variable:
 
122             CONTINUE
                    ALINE = 'Enter output-name for this variable [' //
     &                  VNAME3D( V )( 1 : TRIMLEN( VNAME3D( V ) ) ) //
     &                  '] >> '
                    WRITE( *,95000 ) ALINE( 1:1+TRIMLEN( ALINE ) )
                    READ ( *,93010,IOSTAT=IOS ) ANAME
 
                    IF ( IOS .GT. 0 ) THEN
                        CALL M3WARN( 'M3TPROC', 0, 0,
     &                  'Error reading output-name; please try again' )
                        GO TO 122
                    END IF
 
                IF( ANAME .EQ. BLANK16 ) THEN
                    VNAMEO( NVARS ) = VNAME3D( V )
                ELSE
                    VNAMEO( NVARS ) = ANAME
                END IF
                VNAMEI( NVARS ) = VNAME3D( V )
                UNITS ( NVARS ) = UNITS3D( V )
                VDESC ( NVARS ) = VDESC3D( V )
 
                IF( NVARS .EQ. NVARS3D ) GO TO 199
 
                IF ( NVARS .LT. MXVARS3 )  GO TO  111   !  to head of loop
 
199         CONTINUE        !  end loop getting variables-list for analysis

        END IF  ! If prompting or not
 
        IF ( NVARS .EQ. 0 ) THEN
            CALL M3EXIT( 'M3TPROC', 0, 0,
     &                  'No variables selected', 2 )
        ELSE
            NVARS3D = NVARS
        END IF

C.......   Get starting date and time, and duration:

        IF ( TSTEP .EQ. 0 ) THEN        !  time-independent file

            MESG = 'Input file "' // INAME( 1:TRIMLEN( INAME ) ) //
     &             '" is only one time step- no output written.'
            CALL M3EXIT( 'M3TPROC', 0, 0, MESG, 2 )

        ELSE                            !  time-dependent file

C.......   Prompt for parameters if defaults aren't set by specific env.
C.......   variable setting. Get default settings from the environment.

C...........  Period length
            PERIOD = ENVINT( 'M3TPROC_PLEN', 'Period length', 
     &                        240000, IOS )

            RUNLEN = SEC2TIME( MXREC3D * TIME2SEC( TSTEP ) )
            IF( IOS .NE. 0 ) PERIOD = GETNUM( 1, RUNLEN, PERIOD,
     &        'Enter period length (HHMMSS) for repeating analysis' )

C...........  Start date
            BDATE = ENVINT( 'M3TPROC_BDATE', 'Window start date',
     &                         SDATE, IOS )

            IF( IOS .NE. 0 ) THEN
                BDATE  = GETDATE( BDATE, 
     &               'Enter starting date ' //
     &               'of 1st analysis window (YYYYDDD|YYYYMMDD)' )

            ELSE IF( BDATE .GT. 9999366 ) THEN   ! Convert to Julian

                YR  = BDATE/10000 
                MON = ( BDATE-YR*10000 ) / 100
                DAY = MOD( BDATE-YR*10000, 100 )
                BDATE = YR*1000 + JULIAN( YR, MON, DAY )

            END IF

C...........  Start time
            BTIME = ENVINT( 'M3TPROC_BTIME', 'Window start time',
     &                         STIME, IOS )

            IF( IOS .NE. 0 ) BTIME  = GETNUM( 0, 239999, BTIME,
     &        'Enter starting time of 1st analysis window (HHMMSS)' )

C...........  Duration of analysis window
            WINLEN = ENVINT( 'M3TPROC_WLEN', 'Window duration',
     &                        PERIOD, IOS )

            IF( IOS .NE. 0 ) WINLEN = GETNUM( 1, PERIOD, WINLEN,
     &                'Enter duration of analysis windows (HHMMSS)' )

C...........  Number of periods to process
            RUNLEN = ( TIME2SEC( RUNLEN ) - 
     &                 SECSDIFF( SDATE, STIME, BDATE, BTIME ) )

            PERSEC = TIME2SEC( PERIOD )

            RUNLEN = RUNLEN / PERSEC + 
     &               MOD( RUNLEN, PERSEC ) / TIME2SEC( WINLEN )

            RUNLEN = ENVINT( 'M3TPROC_NPER', 'Number of periods',
     &                        RUNLEN, IOS )

            IF( IOS .NE. 0 ) RUNLEN = GETNUM( 1, RUNLEN, RUNLEN,
     &                          'Enter number of periods to analyze' )

            WSTEPS = TIME2SEC( TSTEP )
            WSTEPS = ( TIME2SEC( WINLEN ) + WSTEPS - 1 ) / WSTEPS

C...........  Type of analysis to perform

            ATYPE = ENVINT( 'M3TPROC_TYPE', 'Type of analysis',
     &                       M3SUM, IOS )


            IF( IOS .NE. 0 ) ATYPE = GETMENU( MAXITEMS, ATYPE, 
     &                        'Enter type of operation to perform', 
     &                        MENUITMS )


        END IF          !  time-independent file, or not

C.......   Build description for the output file, and create accordingly:
C.......   Re-use all but the starting date&time of the input-file description.
        
        SDATE3D = BDATE
        STIME3D = BTIME
        TSTEP3D = PERIOD 

        DO  211  V = 1, NVARS
 
            VNAME3D( V ) = VNAMEO( V )
            UNITS3D( V ) = UNITS ( V )
            VDESC3D( V ) = VDESC ( V )
 
211     CONTINUE
        
        IF ( ARGCNT .EQ. 0 ) THEN
            ONAME = PROMPTMFILE( 'Enter logical name for OUTPUT FILE',
     &                           FSUNKN3, 'OUTFILE', 'M3TPROC' )
        ELSE	!  argcnt = 2:
            IF ( .NOT. OPEN3( ONAME, FSUNKN3, 'M3TPROC' ) ) THEN
                MESG = 'Could not open output file "' // 
     &                 ONAME( 1:TRIMLEN( ONAME ) ) // '"'
                CALL M3EXIT( 'M3TPROC', SDATE, STIME, MESG, 2 )
            END IF
        END IF		!  if argcnt zero, or 2

         
C.......   Process this period in the input file:

        PDATE = BDATE
        PTIME = BTIME

        DO  322  I = 1, RUNLEN

            DO  311  V = 1, NVARS

                JDATE = PDATE
                JTIME = PTIME

                CALL INITAGG( SIZE, LAYER, JDATE, JTIME, INAME,
     &                        VNAMEI( V ), LOGDEV               )

                CALL NEXTIME( JDATE, JTIME, TSTEP )

                DO  301 T = 2, WSTEPS
           
 
                    CALL AGGREG( SIZE, LAYER, JDATE, JTIME, INAME,
     &                           VNAMEI( V ), ATYPE, LOGDEV   )
 

                    CALL NEXTIME( JDATE, JTIME, TSTEP )

301             CONTINUE    !  end loop on window time steps

                CALL OUTAGG( SIZE, LAYER, PDATE, PTIME, ONAME, 
     &                       VNAMEO( V ), ATYPE, WSTEPS, LOGDEV )

311         CONTINUE        !  end loop on variables

            CALL NEXTIME( PDATE, PTIME, PERIOD )
           
322     CONTINUE            !  end loop on analysis periods


        CALL M3EXIT( 'M3TPROC', 0, 0,
     &               'Program  M3TPROC  completed successfully', 0 )


C..............  FORMAT STATEMENTS:  ....................................

C...........   Informational (LOG) message formats... 92xxx

92000   FORMAT ( 5X , A )

92010   FORMAT ( 1X , I5, ':  ', A )

C...........   Formatted file I/O formats............ 93xxx
 
93010   FORMAT( A16 )

C...........   Internal buffering formats............ 94xxx
 
94010   FORMAT ( 100( A, :, 2X, I5, :, 2X ) )

94011   FORMAT ( 3A, I5 )

C...........   Miscellaneous formats................. 95xxx
 
95000   FORMAT ( /5X , A , $ )          !  generic prompt format.

        END

