
C.........................................................................
C Version "@(#)$Header: /env/proj/archive/cvs/ioapi_tools/./ioapi_tools/src/mtxbuild.f,v 1.5 2000/12/21 15:00:18 smith_w Exp $"
C EDSS/Models-3 M3TOOLS.  Copyright (C) 1992-2001 MCNC
C Distributed under the GNU GENERAL PUBLIC LICENSE version 2
C See file "GPL.txt" for conditions of use.
C.........................................................................

        PROGRAM MTXBUILD

C***********************************************************************
C  program body starts at line  96
C
C  DESCRIPTION:
C       Builds a sparse (grid-to-grid transform) matrix from data contained
C	in a "fractions" file.
C
C  PRECONDITIONS REQUIRED:
C       setenv <logical name> <physical (path) name> for the input, 
C       output, and GRIDDESC files.
C	"fractions" file consists of list-formatted (whitespace-delimited)
C	lines with the following fields:
C
C		output grid row number
C		output grid col number
C		input grid row number
C		input grid col number
C		fraction:  AREA(input-cell intersect output-cell)
C			   divided by AREA(input-cell) 
C
C	These lines should  be sorted into lexicographic order
C	in terms of the first four fields.
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       I/O API
C
C  REVISION  HISTORY:
C       Prototype 9/2000 by Carlie J. Coats, Jr., 
C	MCNC Environmental Programs
C       Version 11/2001 by CJC for I/O API Version 2.1
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

      INCLUDE 'PARMS3.EXT'      ! I/O API constants
      INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
      INCLUDE 'IODECL3.EXT'     ! I/O API function declarations


C...........   EXTERNAL FUNCTIONS and their descriptions:

        LOGICAL        DSCGRID
        INTEGER        GETNUM, LBLANK, PROMPTFFILE
        LOGICAL        GETYN
        CHARACTER*16   PROMPTMFILE

        EXTERNAL  DSCGRID, GETNUM, GETYN, LBLANK, 
     &            PROMPTFFILE, PROMPTMFILE
     
C...........   PARAMETERS and their descriptions:

        CHARACTER*80    PROGVER
        DATA PROGVER /
     &  '$Id$'
     &  /


C...........   LOCAL VARIABLES and their descriptions:

        CHARACTER*16    FNAME   !  input fractions file logical name
        CHARACTER*16    MNAME   !  input  matrix   file logical name
        CHARACTER*16    CNAME   !  output coordinate system name
        CHARACTER*16    IGRID   !  input grid name
        CHARACTER*16    OGRID   !  output grid name
        CHARACTER*256   LINE    !  scratch input-line buffer
                
        CHARACTER*256   MESG

        INTEGER         FDEV        !  fractions-file unit number
        INTEGER         LDEV        !  log-device unit number
        INTEGER         STATUS      !  allocation-status
        INTEGER         NCOL1	    !  number of  input-grid rows
        INTEGER         NCOL2	    !  number of output-grid rows
        INTEGER         MROWS	    !  number of sparse-matrix rows
        INTEGER         MCOEF 	    !  number of sparse-matrix coeffs
        INTEGER         V, L, N     !  loop counters

        REAL,    ALLOCATABLE::   CBUF( : )


C***********************************************************************
C   begin body of program MTXBUILD

        LDEV = INIT3()

        WRITE( *, '( 5X, A )' )
     & ' ',
     & 'Program MTXBUILD to build a sparse (grid-to-grid transform)',
     & 'file from an ASCII "fractions" file with lines having the',
     & 'following fields',
     & ' ',
     & '    output grid row number',
     & '    output grid col number',
     & '    input grid row number',
     & '    input grid col number',
     & '    fraction:  AREA(input-cell intersect output-cell)',
     & '             / AREA(input-cell)',
     & 'These lines should  be sorted into lexicographic order',
     & 'relative to the first four fields.',
     & ' ',
     &'See URL  http://www.emc.mcnc.org/products/ioapi/AA.html#tools',
     &' ',
     &'Program copyright (C) 2001 MCNC and released under Version 2',
     &'of the GNU General Public License.  See enclosed GPL.txt, or',
     &'URL  http://www.gnu.org/copyleft/gpl.html',
     &'Comments and questions are welcome and can be sent to',
     &' ',
     &'    envpro@emc.mcnc.org',
     &' ',
     &'    MCNC -- Environmental Modeling Center',
     &'    3021 Cornwallis Rd    P. O. Box 12889',
     &'    Research Triangle Park, NC 27709-2889',
     &' ',
     &'Program version: ' // PROGVER, 
     &' ',
     &'Program release tag: $Name$', 
     &' '

        IF ( .NOT. GETYN( 'Continue with program?', .TRUE. ) ) THEN
            CALL M3EXIT( 'MTXBUILD', 0, 0, 
     &                   'Program terminated at user request', 2 )
        END IF


C...............  Open and "count" input "fractions" file

        MESG = 'Enter logical name for input "fractions" file'
	FDEV = PROMPTFFILE( MESG, .TRUE., .TRUE., 
     &                      'FRACTIONS', 'MTXBUILD' )

	L = 0
	N = 0
        IGRID = ' '
        OGRID = ' '
11      CONTINUE                !  head of "count fractions" loop

	    READ( FDEV, '( A )', END = 22, IOSTAT=STATUS ) LINE
            L = L + 1
            IF ( STATUS .NE. 0 ) THEN
                WRITE( MESG, '(A, I9, 2X, A, I9, 2X, A )' )
     &              'I/O error', STATUS,   
     &              'encountered reading line', N,
     &              'of FRACTIONS file'
                CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 2 )
            END IF
            IF ( LINE( 1:1 ) .NE. '#' ) THEN
                N = N + 1
            ELSE IF ( LINE( 1:7 ) .EQ. '#INGRID'  ) THEN
                V     = 8 + LBLANK( LINE( 8:256 ) )
                IGRID = TRIM( LINE( V:256 ) )
            ELSE IF ( LINE( 1:7 ) .EQ. '#OUTGRID' ) THEN
                V     = 8 + LBLANK( LINE( 8:256 ) )
                OGRID = TRIM( LINE( V:256 ) )
            END IF
            GO TO  11           !  to head of "count fractions" loop

22      CONTINUE                !  exit from "count fractions" loop
	MCOEF = N
        REWIND( FDEV )


C...............  Get output grid description

        IF ( IGRID .EQ. ' ' ) THEN
            CALL GETSTR( 'Enter  input grid name', 'FOO', IGRID )
        ELSE
            MESG = 'Input grid "' // TRIM( IGRID ) // '"'
            CALL M3MSG2( MESG )
        END IF

        IF ( OGRID .EQ. ' ' ) THEN
            CALL GETSTR( 'Enter output grid name', 'BAR', OGRID )
        ELSE
            MESG = 'Output grid "' // TRIM( OGRID ) // '"'
            CALL M3MSG2( MESG )
        END IF

	IF ( .NOT. DSCGRID( IGRID, CNAME, GDTYP3D, 
     &              P_ALP3D, P_BET3D,P_GAM3D, XCENT3D, YCENT3D,
     &              XORIG3D, YORIG3D, XCELL3D, YCELL3D, 
     &              NCOLS3D, NROWS3D, NTHIK3D ) ) THEN

            MESG   = '"' // TRIM( IGRID ) //
     &               '" not found in GRIDDESC file'
            CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 2 )

        END IF
        NCOL1 = NCOLS3D
        
	IF ( .NOT. DSCGRID( OGRID, CNAME, GDTYP3D, 
     &              P_ALP3D, P_BET3D,P_GAM3D, XCENT3D, YCENT3D,
     &              XORIG3D, YORIG3D, XCELL3D, YCELL3D, 
     &              NCOLS3D, NROWS3D, NTHIK3D ) ) THEN

            MESG   = '"' // TRIM( OGRID ) //
     &               '" not found in GRIDDESC file'
            CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 2 )

        END IF
        
        NCOL2 = NCOLS3D
        MROWS = NCOLS3D * NROWS3D

        GDNAM3D = OGRID

        ALLOCATE( CBUF( MROWS + 2 * MCOEF + 1 ), STAT = STATUS )

        IF ( STATUS .NE. 0 ) THEN
            WRITE( MESG, '( A, I10 )' )
     &               'Buffer allocation failed:  STAT=', STATUS
            CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 2 )
        END IF


C...............  Build output sparse matrix file

        NROWS3D = MROWS
        NCOLS3D = MCOEF
        FTYPE3D = SMATRX3
        NVARS3D = 1
        NLAYS3D = 1
        VGTYP3D = IMISS3
        VGTOP3D = BADVAL3
        VGLVS3D( 1 ) = BADVAL3
        VGLVS3D( 2 ) = BADVAL3
        
        SDATE3D = 0
        STIME3D = 0
        TSTEP3D = 0

        VNAME3D( 1 ) = 'COEF'
        VTYPE3D( 1 ) = M3REAL
        UNITS3D( 1 ) = 'n/a'
        VDESC3D( 1 ) = 'Sparse matrix coefficient'
        FDESC3D( 1 ) = 'Sparse transform matrix'
        FDESC3D( 2 ) = '#INGRID  ' // IGRID
        FDESC3D( 3 ) = '#OUTGRID ' // OGRID
        DO  N = 4, MXDESC3
            FDESC3D( N ) = ' '
        END DO
        
        MESG = 'Enter name for output SPARSE MATRIX file'
        MNAME = PROMPTMFILE( MESG, FSUNKN3,
     &                       'MATRIX_FILE', 'MTXBUILD' )


C...............  Read "fractions" file and write out the sparse matrix:

        CALL RDFRAC( FDEV, MCOEF, MROWS, NCOL1, NCOL2, 
     &               CBUF,                      !  "NX" for sparse matrix
     &               CBUF( MROWS+1 ),           !  "IX"
     &               CBUF( MROWS+MCOEF+1 ) )    !  "CX" 

        IF ( .NOT.WRITE3( MNAME, 'ALL', 0, 0, CBUF ) ) THEN
            MESG = 'Could not write out sparse matrix'
            CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 2 )
        END IF

        MESG = 'Successful completion of program MTXBUILD'
        CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 0 )

        END PROGRAM MTXBUILD

C===========================================================================

	SUBROUTINE  RDFRAC( FDEV, MCOLS, MROWS, NCOL1, NCOL2, 
     &                      NX, IX, CX )

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        !!  DESCRIPTION
        !!  	Read the input fractions-file with unit number FDEV
        !!  	into the indicated sparse matrix.
        !!  PRECONDITIONS
        !!	FDEV already open
        !!	FDEV consists of list-formatted lines sorted into
        !!	lexicographic order by the first four fields
        !!	The fields are:
	!!        	input grid row number
	!!        	input grid col number
	!!        	output grid row number
	!!        	output grid col number
	!!        	fraction:    AREA(input-cell intersect output-cell)
	!!        		   / AREA(input-cell)
        !!
        !!  HISTORY
        !!      Prototype 9/2000 by Carlie J. Coats, Jr., MCNC-EP
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 

        IMPLICIT NONE

        !!  Arguments:

        INTEGER, INTENT(IN) ::   FDEV        !  fractions-file unit number
        INTEGER, INTENT(IN) ::   MCOLS
        INTEGER, INTENT(IN) ::   MROWS
        INTEGER, INTENT(IN) ::   NCOL1
        INTEGER, INTENT(IN) ::   NCOL2
        INTEGER, INTENT(OUT)::   NX( MROWS )
        INTEGER, INTENT(OUT)::   IX( MCOLS )
        REAL,    INTENT(OUT)::   CX( MCOLS )


        !!  Local Variables:

        INTEGER         L, J, N, R, ROW
        INTEGER         STATUS
        INTEGER         LAST
        INTEGER         ICOL, IROW, JCOL, JROW
        REAL            FRAC
        CHARACTER*256   LINE, MESG

        !!..................  body of RDFRAC  ..............................

        DO R = 1, MROWS
            NX( R ) = 0
        END DO

        DO R = 1, MCOLS
            IX( R ) = 0
            CX( R ) = 0.0
        END DO

        L = 0
        J = 0
        

111     CONTINUE	!  loop:  skip header lines

            L = L + 1
            READ( FDEV, '( A )', IOSTAT=STATUS ) LINE
            IF ( STATUS .NE. 0 ) THEN
                 WRITE( MESG, '(A, I9, 2X, A, I9, 2X, A )' )
     &              'I/O error', STATUS,   
     &              'encountered reading line', L,
     &              'of FRACTIONS file'
                 CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 2 )
            END IF
            IF ( LINE( 1:1 ) .EQ. '#' ) GO TO 111

        READ( LINE, *, IOSTAT=STATUS ) 
     &          IROW, ICOL, JROW, JCOL, FRAC
        IF ( STATUS .NE. 0 ) THEN
            WRITE( MESG, '(A, I9, 2X, A, I9, 2X, A )' )
     &          'I/O error', STATUS,   
     &          'encountered parsing line', L,
     &          'of FRACTIONS file'
             CALL M3EXIT( 'MTXBUILD', 0, 0, MESG, 2 )
        END IF
        ROW = ICOL + ( IROW - 1 )*NCOL2
        
        DO  R = 1, MROWS	! loop on matrix rows

            N = 0

122         CONTINUE		!  loop reading matrix coeffs for this row

                IF ( ROW .GT. R ) GO TO 133

                N = N + 1
                J = J + 1
                IX( J ) = JCOL + ( JROW - 1 )*NCOL1
                CX( J ) = FRAC

                L = L + 1
                READ( FDEV, '( A )', END=133, IOSTAT=STATUS ) LINE
                IF ( STATUS .NE. 0 ) THEN
                    WRITE( MESG, '(A, I9, 2X, A, I9, 2X, A )' )
     &                  'I/O error', STATUS,   
     &                 'encountered reading line', L,
     &                  'of FRACTIONS file'
                     CALL M3EXIT( 'MTXBUILD/RDFRAC', 0, 0, MESG, 2 )
                END IF
                READ( LINE, *, IOSTAT=STATUS ) 
     &                  IROW, ICOL, JROW, JCOL, FRAC

                IF ( STATUS .NE. 0 ) THEN
                    WRITE( MESG, '(A, I9, 2X, A, I9, 2X, A )' )
     &                  'I/O error', STATUS,   
     &                 'encountered parsing line', L,
     &                  'of FRACTIONS file'
                     CALL M3EXIT( 'MTXBUILD/RDFRAC', 0, 0, MESG, 2 )
                END IF
                ROW = ICOL + ( IROW - 1 )*NCOL2

                GO TO 122		!  to head of loop reading matrix coeffs

133         CONTINUE			!  exit from loop reading matrix coeffs

            NX( R ) = N

        END DO			!  end loop on matrix rows

        RETURN

        END SUBROUTINE RDFRAC
